import json
from datetime import date
from datetime import datetime
import array


class MatlabDataEncoder(json.JSONEncoder):
    def default(self, field):

        if isinstance(field, datetime):
            return field.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(field, date):
            return field.strftime('%Y-%m-%d')
        elif isinstance(field, array.array):
            return field.tolist()
        elif isinstance(field, memoryview):
            return field.tolist()
        else:
            return json.JSONEncoder.default(self, field)


class DateTimeEncode(json.JSONEncoder):
    def default(self, field):
        if isinstance(field, datetime):
            return field.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(field, date):
            return field.strftime('%Y-%m-%d')
        else:
            return json.JSONEncoder.default(self, field)


import os
import io
from ..utils import fileLoad
import gzip
import yaml
import datetime
import pytz
import random

utc_tz = pytz.timezone('UTC')


class Storage(object):
    """
        消息存储类，

        用于存储接收到的消息数据，并提供数据保存、加载两个数据本地化函数

        提供通过数据类型获取数据，通过数据位置获取数据、和获取当前数据长度的函数

    """
    def __init__(self,
                 key,
                 name,
                 job,
                 config,
                 revision,
                 model,
                 messages=None,
                 finished=None,
                 **kwargs):
        """
            初始化
        """
        self.message = messages if messages is not None else []
        self.taskId = key
        self.end = False
        self.name = name if name is not None else ''.join(
            random.sample([
                'z', 'y', 'x', 'w', 'v', 'u', 't', 's', 'r', 'q', 'p', 'o',
                'n', 'm', 'l', 'k', 'j', 'i', 'h', 'g', 'f', 'e', 'd', 'c',
                'b', 'a'
            ], 5))
        self.job = job
        self.config = config
        self.revision = revision
        self.started = datetime.datetime.now(tz=utc_tz).isoformat()
        self.finished = finished
        self.model = model

    def storeMessage(self, message):
        """
            消息存储 db.storeMessage(message)

            :params message: 需要存储的消息
        """
        self.message.append(message)

    @staticmethod
    def dump(result, file):
        '''
            保存结果到本地文件

            :params: file 保存文件的目录

            >>> Result.dump(result,file)
            {...}
        '''

        data = {
            'key': result.taskId,
            'name': result.name,
            'model': result.model,
            'job': result.job,
            'config': result.config,
            'status': 'succeeded',
            'revision': result.revision,
            'messages': result.message,
            'started': result.started,
            'finished': result.finished,
            'queued': datetime.datetime.now(),
            'legacy': []
        }
        data = yaml.dump(data)
        with gzip.open(file, 'w') as output:
            with io.TextIOWrapper(output, encoding='utf-8') as enc:
                enc.write(data)

    @staticmethod
    def load(file):
        """
            加载本地结果文件

            :params: file 文件目录

            :returns: 返回一个项目实例

            >>> result = Result.load('C:\\Users\\dps-dm\\cloudpss-sdk\\result\\424111.cjob')

        """

        if not os.path.exists(file):
            raise FileNotFoundError('未找到文件')
        data = fileLoad(file)
        result = Storage(**data)
        return result

    def getMessagesByType(self, type):
        """
            获取指定类型的消息数据

            :params type: 数据类型

            :returns: 对应类型的数据数组

            >>> message= db.getMessagesByType('log')
        """

        result = []
        for val in self.message:
            if val['type'] == type:
                result.append(val)
        return result

    def getMessagesByKey(self, key):
        """
            获取指定 key 的消息数据

            :params key: 数据key

            :returns: 对应 key 的数据数组

            >>> message= db.getMessagesByKey('log')
        """

        result = []
        for val in self.message:
            if val.get('key', None) == key:
                result.append(val)
        return result

    def getMessageLength(self):
        """
            获取指定消息的长度

            :returns: 数据长度

            >>> length= db.getMessageLength()
        """
        return len(self.message)

    def getMessage(self, index):
        """
            获取指定位置的消息数据

            :params index: 数据的位置信息

            :returns: 消息数据

            >>> message= db.getMessage(1)
        """
        return self.message[index]



import os,sys
import time

import socketio
import threading
import struct

                         
if __name__ == '__main__':
   a = [1.21,2.22,3.33]
   x = struct.pack('d'*len(a),*a)
   print(x)
   print(len(x))
   
   t = struct.unpack('d'*int(len(x)/8),x)
   
        
   print(t)


from .component import Component
from typing import Optional
import uuid


class DiagramImplement(object):
    """
        拓扑实现
    """

    def __init__(self, diagram: dict = {}):
        """
            初始化
        """
        for k, v in diagram.items():
            if k == 'cells':
                self.__dict__[k] = v
                for key, val in v.items():
                    v[key] = Component(val)
            else:
                self.__dict__[k] = v

    def __getitem__(self, attr):
        return super(DiagramImplement, self).__getattribute__(attr)

    def toJSON(self):
        """
            类对象序列化为 dict
            :return: dict

            >>>> diagram.toJSON()
        """
        cells = {}
        for key, val in self.cells.items():
            cells[key] = val.toJSON()
        diagram = {**self.__dict__, 'cells': cells}
        return diagram

    def getAllComponents(self):
        """
            获取所有元件

            :return: dict<Component>

            >>>> diagram.getAllComponents()
        """

        return self.cells
    
    def addComponent(
        self,
        definition: str,
        label: str,
        args: dict = {},
        pins: dict = {},
        canvas: Optional[str] = None,
        position: Optional[dict] = None,
        size: Optional[dict] = None,
    ) -> Component:
        """
        添加元件

        :param definition 元件定义， 连接线没有definition
        :param label 元件标签
        :param args 元件参数数据，连接线没有参数数据
        :param pins 元件引脚数据，连接线没有引脚数据
        :param canvas 元件所在图纸数据
        :param position 元件位置数据, 连接线没有位置数据
        :param size 元件大小数据，连接线没有大小数据

        :return: Component

        >>>> diagram.addComponent(args)
        """
        id = "comp_" + str(uuid.uuid4())
        shape = "diagram-component"
        definition = definition
        label = label or definition
        args = args.copy()
        pins = pins.copy()
        props = {"enabled": True}
        context = {}
        canvas or self.canvas[0].get("key", "canvas_0")
        position = position.copy() if position else {"x": 0, "y": 0}
        size = size.copy() if size else None
        zIndex = 0
        style = {
            "--fill": "var(--spectrum-global-color-gray-100)",
            "--fill-opacity": 1,
            "--font-family": "var(--spectrum-global-font-family-base, Arial, Helvetica, sans-serif)",
            "--stroke": "var(--spectrum-global-color-gray-900)",
            "--stroke-opacity": 1,
            "--stroke-width": 2,
            "--text": "var(--spectrum-global-color-gray-900)",
            "--text-opacity": 1,
        }
        diagram = {
            "id": id,
            "shape": shape,
            "definition": definition,
            "label": label,
            "args": args,
            "pins": pins,
            "props": props,
            "context": context,
            "canvas": canvas,
            "position": position,
            "size": size,
            "zIndex": zIndex,
            "style": style,
        }
        component = Component(diagram)
        self.cells[id] = component
        return component

    def removeComponent(self, key: str) -> bool:
        """
        删除元件

        :param key: str
        :return: bool

        >>>> diagram.removeComponent(key)
        """
        component = self.cells.get(key)
        if not component:
            return False
        position = component.position.copy()
        del self.cells[key]
        for edge in self.cells.values():
            if edge.shape == "diagram-edge":
                if edge.source.get("cell") == key:
                    edge.source = position.copy()
                    position["x"] += 5
                    position["y"] += 5

                if edge.target.get("cell") == key:
                    edge.target = position.copy()
                    position["x"] += 5
                    position["y"] += 5
        return True

    def updateComponent(self, key: str, args: dict) -> bool:
        """
        更新元件

        :param key: str
        :param args: dict
        :return: bool

        >>>> diagram.updateComponent(key)
        """
        component = self.cells.get(key)
        if not component:
            return False
        for k, v in self.cells.items():
            if k == key:
                v.__dict__.update(args)
                return True



from .result import Result
import re
import copy
class IESLabTypicalDayResult(Result):
    def __init__(self, *args, **kwargs):
        """
            初始化
        """
        Result.__init__(self, *args, **kwargs)
        self.__plotIndex = 0
        self.__typicalIndex = 0
        self.__type_list =['电负荷', '热负荷','冷负荷','氢负荷','散射辐射', '直射辐射',  '天顶角', '环境温度', '土壤温度', '10m风速', '50m风速']
        self.result = {'TypicalMonth': [],'TypicalDay': []}
        for i in range(12):
            self.result['TypicalMonth'].append({'月份': int, '持续天数': [],'电负荷': [], '热负荷': [],'冷负荷':[],'氢负荷':[],'散射辐射': [], '直射辐射': [],'天顶角': [],'环境温度': [], '土壤温度': [], '10m风速': [], '50m风速': []})
        

    def __readPlotResult(self):
        length = self.db.getMessageLength()
        if (length > self.__plotIndex):
            for num in range(self.__plotIndex, length):# update TypicalMonth
                val = self.db.getMessage(num)
                if val['type'] == 'plot':
                    key_list = re.split('-month',val['key'])#分别为类型和月份
                    self.result['TypicalMonth'][int(key_list[1])-1]['月份'] = int(key_list[1])
                    if key_list[0] == '散射辐射':#每个月各个典型日的天数
                        typicalNum = len(val['data']['traces'])
                        for i in range(typicalNum):
                            self.result['TypicalMonth'][int(key_list[1])-1]['持续天数'].append(int(re.findall('\d+',val['data']['traces'][i]['name'])[1]))
                    # 每个月各类型数据的各个典型日的数据，由于部分月份可能没有电冷热负荷，某月的某个典型日可能缺少冷热负荷
                    for i in range(typicalNum):
                        self.result['TypicalMonth'][int(key_list[1])-1][key_list[0]].append([])
                    for i in range(len(val['data']['traces'])):
                        self.result['TypicalMonth'][int(key_list[1])-1][key_list[0]][int(re.findall('\d+',val['data']['traces'][i]['name'])[0])-1] = copy.deepcopy(val['data']['traces'][i]['y'])
            self.__plotIndex = length
            # update TypicalDay based on TypicalMonth
            for m in range(12):
                for i in range(len(self.result['TypicalMonth'][m]['持续天数'])):
                    self.result['TypicalDay'].append({'info':{'typicalDayID': int, 'name': str, 'duration': int, 'maxElectricalLoad': 0.0, 'maxHeatLoad': 0.0, 'maxCoolLoad': 0.0}, 
                                                      'data': {'电负荷': [], '热负荷': [],'冷负荷':[],'氢负荷':[],'散射辐射': [], '直射辐射': [],'天顶角': [],'环境温度': [], '土壤温度': [], '10m风速': [], '50m风速': []}})
                    self.result['TypicalDay'][-1]['info']['typicalDayID'] = self.__typicalIndex
                    self.result['TypicalDay'][-1]['info']['name'] = str(m+1) + '月典型日' + str(i+1)
                    self.result['TypicalDay'][-1]['info']['duration'] = self.result['TypicalMonth'][m]['持续天数'][i]
                    if self.result['TypicalMonth'][m]['电负荷']:
                        if self.result['TypicalMonth'][m]['电负荷'][i]:
                            self.result['TypicalDay'][-1]['info']['maxElectricalLoad'] = max(self.result['TypicalMonth'][m]['电负荷'][i])
                    if self.result['TypicalMonth'][m]['氢负荷']:
                        if self.result['TypicalMonth'][m]['氢负荷'][i]:
                            self.result['TypicalDay'][-1]['info']['maxHydrogenLoad'] = max(self.result['TypicalMonth'][m]['氢负荷'][i])
                    if self.result['TypicalMonth'][m]['热负荷']:
                        if self.result['TypicalMonth'][m]['热负荷'][i]:
                            self.result['TypicalDay'][-1]['info']['maxHeatLoad'] = max(self.result['TypicalMonth'][m]['热负荷'][i])
                    if self.result['TypicalMonth'][m]['冷负荷']:
                        if self.result['TypicalMonth'][m]['冷负荷'][i]:
                            self.result['TypicalDay'][-1]['info']['maxCoolLoad'] = max(self.result['TypicalMonth'][m]['冷负荷'][i])
                    for type_i in self.__type_list:
                        if self.result['TypicalMonth'][m][type_i]:
                            self.result['TypicalDay'][-1]['data'][type_i] = self.result['TypicalMonth'][m][type_i][i]
                    self.__typicalIndex += 1
            
    def GetTypical(self):
        '''
            获取所有的 GetTypical 典型日数据

            >>> result.GetTypical()
            {...}
        '''
        self.__readPlotResult()
        return self.result['TypicalDay']
        

    def GetTypicalDayNum(self):
        '''
            获取当前result的典型日数量
            
            :return: int类型，代表典型日数量
        '''
        self.__readPlotResult()
        return self.__typicalIndex
        
    def GetTypicalDayInfo(self,dayID):
        '''
            获取dayID对应典型日的基础信息
            
            :params: dayID int类型，表示典型日的ID，数值位于 0~典型日数量 之间
            
            :return: dict类型，代表典型日的基础信息，包括典型日所代表的日期范围、典型日的名称等
        '''
        self.__readPlotResult()
        return self.result['TypicalDay'][dayID]['info']
        

    def GetTypicalDayCurve(self,dayID, dataType):
        '''
            获取dayID对应典型日下dataType参数的时序曲线
            
            :params: dayID int类型，表示典型日的ID，数值位于 0~典型日数量 之间
            :params: dataType enum类型，标识辐照强度、环境温度、土壤温度、建筑物高度风速、风机高度风速、电负荷、热负荷、冷负荷的参数类型
            
            :return: list<float>类型，代表以1h为时间间隔的该参数的日内时序曲线
        '''
        self.__readPlotResult()
        return self.result['TypicalDay'][dayID]['data'][dataType]
        
        
    def GetTypicalMonth(self):
        '''
            获取所有的 GetTypicalMonth 数据
            
            >>> result.GetTypicalMonth()
            
            :return: list<dict>类型，代表各月各类型的典型日数据
        '''
        self.__readPlotResult()
        return self.result['TypicalMonth']
    
    def GetTypicalMonthNum(self,monthID):
        '''
            获取第monthID月各类型的典型日数据

            >>> result.GetTypicalMonthNum()
            
            :params: monthID int类型，表示典型月的ID，数值位于 1-12 之间

            :return: dict类型，代表第monthID月各类型的典型日数据
            {...}
        '''
        self.__readPlotResult()
        return self.result['TypicalMonth'][monthID-1]
        
    
    def GetTypicalMonthCurve(self,monthID, dataType):
        '''
            获取dayID对应典型日下dataType参数的时序曲线
            
            :params: monthID int类型，表示典型月的ID，数值位于 1-12 之间
            :params: dataType enum类型，标识辐照强度、环境温度、土壤温度、建筑物高度风速、风机高度风速、电负荷、热负荷、冷负荷的参数类型
            
            :return: list<list>类型，代表以1h为时间间隔的该参数的典型日内时序曲线
        '''
        self.__readPlotResult()
        return self.result['TypicalMonth'][monthID-1][dataType]


from cProfile import run
import threading
import json
import time
import random
from .receiver import Receiver
from .result import IESLabSimulationResult, PowerFlowResult, EMTResult, Result, IESResult
from .IESLabPlanResult import IESLabPlanResult
from .IESLabEvaluationResult import IESLabEvaluationResult
from .IESLabTypicalDayResult import IESLabTypicalDayResult
from .storage import Storage
from ..utils import request
from typing import  TypeVar,  Generic

RECEIVER = {
    'default': Receiver,
}
T = TypeVar('T', Result, EMTResult, PowerFlowResult, IESResult,
            IESLabSimulationResult, IESLabPlanResult, IESLabEvaluationResult,IESLabTypicalDayResult)

IES_LAB_RESULT = {
    'function/ieslab/plan': IESLabPlanResult,
    'function/ieslab/evaluation': IESLabEvaluationResult
}

RESULT_DB = {
    'function/CloudPSS/emtp': EMTResult,
    'function/CloudPSS/emtps': EMTResult,
    'function/CloudPSS/sfemt': EMTResult,
    'function/CloudPSS/power-flow': PowerFlowResult,
    'function/CloudPSS/ies-simulation': IESResult,
    'function/CloudPSS/ies-optimization': IESResult,
    'function/ies/ies-optimization': IESResult,
    'function/CloudPSS/three-phase-powerFlow': PowerFlowResult,
    'function/ies/ies-simulation': IESLabSimulationResult,
    'function/ies/ies-gmm':IESLabTypicalDayResult,
    'function/CloudPSS/ieslab-simulation': IESLabSimulationResult,
    'function/CloudPSS/ieslab-gmm':IESLabTypicalDayResult,
    'function/CloudPSS/ieslab-optimization': IESResult,
}

class Runner(Generic[T]):
    def __init__(self, taskId, name, job, config, revision, modelRid,
                 **kwargs):
        self.taskId = taskId
        self.db = Storage(taskId, name, job, config, revision, modelRid)
        rid =job['rid'].replace('job-definition/','function/').replace('/cloudpss/','/CloudPSS/')
        result = RESULT_DB.get(rid, Result)
        self.result: T = result(self.db)
        self.receiver = kwargs.get('receiver', None)

    def __listenStatus(self):
        if self.receiver is None:
            return False
        if self.receiver.status() == -1:
            raise Exception(self.receiver.error)
        return self.receiver.isOpen

    def status(self):
        """
        运行状态
        :return: 运行状态  0/1/-1 1 表示正常结束，0 表示运行中， -1 表示数据接收异常


        >>>> runner.status()  
        """
        if self.receiver is None:
            raise Exception('not find receiver')
        return self.receiver.status()

    def __listen(self, **kwargs):

        receiver = kwargs.get('RECEIVER', 'default')
        receiverclass = None
        if type(receiver) is str:
            if receiver not in RECEIVER:
                receiverclass = RECEIVER['default']
            else:
                receiverclass = RECEIVER[receiver]
        if receiverclass is None:
            raise Exception('not find receiver')
        self.receiver = receiverclass(self.taskId, self.db, **kwargs)
        self.receiver.connect()

    def terminate(self):
        """
        结束当前运行的算例

        """
        r = request("DELETE", 'api/simulation/runner/' + str(self.taskId))

    @staticmethod
    def create(revisionHash, job, config, name=None, rid='', **kwargs):
        '''
            创建一个运行任务

            :params: revision 项目版本号
            :params: job 调用仿真时使用的计算方案，为空时使用项目的第一个计算方案
            :params: config 调用仿真时使用的参数方案，为空时使用项目的第一个参数方案
            :params: name 任务名称，为空时使用项目的参数方案名称和计算方案名称
            :params: rid 项目rid，可为空

            :return: 返回一个运行实例

            >>> runner = Runner.runRevision(revision,job,config,'')
        '''
        taskId = str(int(time.time() * random.random()))

        runner = Runner(taskId, name, job, config, revisionHash, rid, **kwargs)
        event = threading.Event()
        thread = threading.Thread(target=runner.__listen, kwargs=kwargs)
        thread.setDaemon(True)
        thread.start()
        payload = {
            'args': config['args'],
            'job': job,
            'implement': kwargs.get('topology', None)
        }
        while not runner.__listenStatus():
            time.sleep(0.1)
        r = request('POST',
                    'api/simulation/runner/' + revisionHash + '/' +
                    str(taskId),
                    data=json.dumps(payload))

        return runner


class HttpRunner(Runner[T]):
    def __init__(self, job, simulationId, **kwargs):
        self.simulationId = simulationId
        self.job = job
        self.__taskId = self.__getLastTask()
        result = IES_LAB_RESULT.get(job.get('rid', ''), IESLabPlanResult)
        self.result: T = result(self.simulationId, self.__taskId, **kwargs)

    def __getLastTask(self):
        r = request('GET',
                    'api/ieslab-plan/taskmanager/getSimuLastTasks',
                    params={'simuid': self.simulationId})
        result = json.loads(r.text)
        return result['data'].get('task_id', None)

    def status(self):
        if self.__taskId is None:
            return False
        return self.result.status()  # type: ignore



from ..utils import request, fileLoad, graphql_request
import json
import time, datetime
import copy


class DataManageModel(object):
    _kindUrlMap = {}
    _kindNameMap = {}
    _weatherUrl = ''
    _baseUri = ''
    _itemDataMap={}
    _kindItemDataMap={}
    _kindIdMap={}
    def __init__(self, simulationId):
        self.simulationId = simulationId
        self._getAllData()
    def _fetchItemData(self, url, kind):
        '''
            私有方法，获取kind类型对应所有数据项的列表

            :params: url string类型，request请求对应的url链接
            
            :params: kind string类型，数据的种类标识，比如PhotovoltaicSys, WindPowerGenerator, GasTurbine等等
            
            :return: list类型，返回该种类下所有数据项的的列表
        '''
        r = request('GET',
                    url,
                    params={
                        "simu_id": self.simulationId,
                        "kind": kind
                    })
        str = url.split('/')[-2]
        if(str in ['typhoon', 'rainfall', 'earthquake', 'extremeCold'] and r.status_code == 404):
            return {}
        else :
            data = json.loads(r.text)
            return data['results']

    def _deleteItemData(self, url):
        '''
            私有方法，删除url链接对应的数据
            
            :params: url string类型，request请求对应的url链接
            
            :return: 无
        '''
        r = request('DELETE',
                    url)
        return r.text
    def _saveItemData(self, url, data):
        '''
            私有方法，保存url链接对应的data数据
            
            :params: url string类型，request请求对应的url链接

            :params: data dict类型，表示添加的数据内容，其数据结构应满足对应数据项的结构要求
            
            :return: dict类型，表示成功添加的数据内容
        '''
        r = request('POST', url, data=json.dumps(data))
        dataList = json.loads(r.text)
        return dataList

    def _updateItemData(self, url, data):
        '''
            私有方法，更新url链接对应的data数据
            
            :params: url string类型，request请求对应的url链接

            :params: data dict类型，表示添加的数据内容，其数据结构应满足对应数据项的结构要求
            
            :return: dict类型，表示成功更新的数据内容
        '''
        r = request('PATCH', url, data=json.dumps(data))
        dataList = json.loads(r.text)
        return dataList
    
    def _getAllData(self):
        '''
            私有方法，获取数据管理模块全部数据，函数初始化时调用
            
            :return: 无
        '''
        list = ['thermalLoads', 'electricLoads', 'fuels', 'typhoon', 'rainfall', 'earthquake', 'extremeCold', 'HydrogenLoad']
        for kind,value in self._kindUrlMap.items():
            try:
                if kind in list:
                    dataList = self._fetchItemData(self._kindUrlMap[kind], None)
                else:
                    dataList = self._fetchItemData(self._kindUrlMap[kind], kind)
            except Exception as e:
                pass
            self._kindItemDataMap[kind]=dataList
            for val in dataList:
                self._itemDataMap[str(val['timeid'])]=val
                self._itemDataMap[str(val['id'])]=val
                self._kindIdMap[str(val['timeid'])]=kind
                self._kindIdMap[str(val['id'])]=kind

    def GetDataItem(self, ID: str):
        '''
            获取ID对应的元素信息
            
            :params: ID string类型，代表数据项的标识符，可以在所有类型的数据项中实现唯一标识
            
            :return: dict类型，为源数据的引用，返回该数据项的信息
        '''

        data = self._itemDataMap.get(str(ID),None)
        
        assert (data is not None), "找不到数据"

        return copy.deepcopy(data)

    def GetItemList(self, dataType):
        '''
            获取dataType类型对应所有数据项的列表
    
            :params: dataType enum类型，数据的种类标识，包含："光伏": "PhotovoltaicSys","风机": "WindPowerGenerator","燃气轮机": "GasTurbine","热泵": "HeatPump","燃气锅炉": "GasBoiler","热管式太阳能集热器": "HPSolarCollector","电压缩制冷机": "CompRefrg","吸收式制冷机": "AbsorptionChiller","蓄电池": "Battery","储水罐": "WaterTank","变压器": "Transformer","传输线": "TransferLine","模块化多电平变流器": "MMC","离心泵": "CentrifugalPump","管道": "Pipe","采暖制冷负荷": "thermalLoads","电负荷": "electricLoads","燃料": "fuels","热": "HVACHeating","冷": "HVACCooling","常数电价": "常数电价","分时电价": "分时电价","阶梯电价": "阶梯电价","分时阶梯电价": "分时阶梯电价"
    
            :return: list类型，返回该种类下所有数据项的列表
        '''
        assert (dataType in self._kindNameMap
                or dataType in self._kindUrlMap), "数据类型不存在"
        kind = self._kindNameMap.get(dataType, dataType)
        return copy.deepcopy(self._kindItemDataMap[kind])

    def AddDataItem(self, dataType, data):
        '''
            向dataType类型的数据库中添加内容为data的数据项
    
            :params: dataType enum类型，数据的种类标识，包含："光伏"、"风机"、"燃气轮机"、"热泵"、"燃气锅炉"、"热管式太阳能集热器"、"电压缩制冷机"、"吸收式制冷机"、"蓄电池"、"储水罐"、"变压器"、"传输线"、"模块化多电平变流器"、"离心泵"、"管道"、"采暖制冷负荷"、"电负荷"、"燃料"、"热"、"冷"、"常数电价"、"分时电价"、"阶梯电价"、"分时阶梯电价"
    
            :params: data dict类型，表示添加的数据内容，其数据结构应满足对应数据项的结构要求
    
            :return: string类型，返回新添加数据项的ID，如果数据结构不满足要求，应当抛出异常
        '''
        assert (dataType in self._kindNameMap
                or dataType in self._kindUrlMap), "数据类型不存在"
        kind = self._kindNameMap.get(dataType, dataType)
        r = copy.deepcopy(data)
        if r['id'] is not None:
            del r['id']
        if r['timeid'] is not None:
            del r['timeid']
        try:
            self._saveItemData(self._kindUrlMap[kind], r)
            self._kindIdMap[str(data['timeid'])]=kind
            self._kindIdMap[str(data['id'])]=kind
            self._itemDataMap[str(data['timeid'])]=data
            self._itemDataMap[str(data['id'])]=data
            if kind in ['thermalLoads', 'electricLoads', 'fuels']:
                dataList = self._fetchItemData(self._kindUrlMap[kind], None)
                self._kindItemDataMap[kind] = dataList
                return dataList[-1]['id']
            else:
                dataList = self._fetchItemData(self._kindUrlMap[kind], kind)
                self._kindItemDataMap[kind] = dataList
                return dataList[-1]['id']
        except Exception as e:
            raise Exception(str(e))

    def UpdateDataItem(self, ID: str, data):
        '''
            更新数据库ID对应数据项"光伏"、"风机"、"燃气轮机"、"热泵"、"燃气锅炉"、"热管式太阳能集热器"、"电压缩制冷机"、"吸收式制冷机"、"蓄电池"、"储水罐"、"变压器"、"传输线"、"模块化多电平变流器"、"离心泵"、"管道"、"采暖制冷负荷"、"电负荷"、"燃料"、"热"、"冷"、"常数电价"、"分时电价"、"阶梯电价"、"分时阶梯电价"数据
            
            :params: id string类型，代表数据项的标识符，可以在所有类型的数据项中实现唯一标识
            :params: data dict类型，表示添加的数据内容，其数据结构应满足对应数据项的结构要求
            
            :return: bool 类型，返回True 更新成功
        '''
        try:
            kind = self._kindNameMap.get(data['kind'], data['kind'])
            if kind is not None:
                url=self._kindUrlMap[kind] + '/' + str(ID) + '/'
                ret = self._updateItemData(url, data)
                for i in range(len(self._kindItemDataMap[kind])):
                    if str(self._kindItemDataMap[kind][i]['id']) == str(ID):
                        self._kindItemDataMap[kind][i] = data
                        break
                    else:
                        pass
                return True
            else:
                raise Exception('传入的数据缺乏kind类型')
        except Exception as e:
            raise Exception(str(e))

    def DeleteDataItem(self, ID: str):
        '''
            向数据库中删除ID对应数据项"光伏"、"风机"、"燃气轮机"、"热泵"、"燃气锅炉"、"热管式太阳能集热器"、"电压缩制冷机"、"吸收式制冷机"、"蓄电池"、"储水罐"、"变压器"、"传输线"、"模块化多电平变流器"、"离心泵"、"管道"、"采暖制冷负荷"、"电负荷"、"燃料"、"热"、"冷"、"常数电价"、"分时电价"、"阶梯电价"、"分时阶梯电价"数据
    
            :params: ID string类型，代表数据项的标识符，可以在所有类型的数据项中实现唯一标识
            
            :return: bool 类型，删除是否成功，如果ID错误，抛出异常
        '''
        data = self._itemDataMap.get(str(ID),None)
        kind = self._kindIdMap.get(str(ID))
        if data is None or kind is None:
            raise Exception('id错误，未找到该id资源')
        else:
            url=self._kindUrlMap[kind]
            self._deleteItemData(url+str(data['id'])+'/')
            del self._itemDataMap[str(data['timeid'])]
            del self._itemDataMap[str(data['id'])]
            dataList=self._kindItemDataMap[kind]
            for i in range(len(dataList)):
                if dataList[i]['id'] == data['id']:
                    del dataList[i]
                    break
            return True

    def SetProjectPosition(self, longitude, latitude):
        '''
            将项目的经纬度位置坐标设置为(longitude, latitude)
            
            :params: longitude float类型，表示经度，范围为气象数据源的经度范围
            :params: latitude float类型，表示纬度，范围为气象数据源的纬度范围
        '''
        if (float(longitude) > 180 or float(longitude) < -180
                or float(latitude) > 90 or float(latitude) < -90):
            raise Exception('经纬度坐标不存在')
        else:
            r = request('GET',
                        self._baseUri + 'rest/weather_param/',
                        params={"simu": self.simulationId})
            param = json.loads(r.text)
            if (len(param["results"]) == 0):
                r = request('POST',
                            self._baseUri + 'rest/weather_param/',
                            data=json.dumps({
                                "lat": latitude,
                                "lng": longitude,
                                "simu": self.simulationId,
                                "simu_id": self.simulationId,
                            }))
            else:
                id = param['results'][0]['id']
                r = request('PUT',
                            self._baseUri + 'rest/weather_param/' + str(id) +
                            '/',
                            data=json.dumps({
                                "id": id,
                                "lat": latitude,
                                "lng": longitude,
                                "simu": self.simulationId,
                                "simu_id": self.simulationId,
                            }))
                r = request('GET',
                            self._baseUri + 'load_weather/',
                            params={
                                "lat": latitude,
                                "lng": longitude,
                                "simu": self.simulationId,
                                "simu_id": self.simulationId,
                            })

    def GetAtmosData(self, startDate, endDate):
        '''
            获取在startDate到endDate之间的气象数据
            
            :params: startDate dateTime类型，表示开始时间
            :params: endDate dateTime类型，表示结束时间
            
            :return: list<dict>类型，为源数据的引用，返回当前项目位置对应时间范围内的气象数据序列，每个元素用字典进行表示，字典的key即区分不同的气象数据项（如风速、太阳辐照等）以及标识当前时间点
        '''
        sDate = datetime.date(*map(int, startDate.split('-')))
        eDate = datetime.date(*map(int, endDate.split('-')))
        res = (eDate - sDate).days * 86400 + (eDate - sDate).seconds
        if res < 0:
            raise Exception('超出有效时间范围')
        else:
            r = request('GET',
                        self._weatherUrl,
                        params={
                            "time_after": startDate,
                            "time_before": endDate,
                            "sid": self.simulationId,
                        })
            weatherData = json.loads(r.text)
            return weatherData['results']


class IESSimulationDataManageModel(DataManageModel):
    _baseUri = 'api/ieslab-simulation/'
    _weatherUrl = 'api/ieslab-simulation/rest/weather_data/'
    _kindNameMap = {
        "光伏": "PhotovoltaicSys",
        "风机": "WindPowerGenerator",
        "燃气轮机": "GasTurbine",
        "热泵": "HeatPump",
        "燃气锅炉": "GasBoiler",
        "热管式太阳能集热器": "HPSolarCollector",
        "电压缩制冷机": "CompRefrg",
        "吸收式制冷机": "AbsorptionChiller",
        "蓄电池": "Battery",
        "储水罐": "WaterTank",
        "变压器": "Transformer",
        "传输线": "TransferLine",
        "模块化多电平变流器": "MMC",
        "离心泵": "CentrifugalPump",
        "管道": "Pipe",
        "采暖制冷负荷": "thermalLoads",
        "电负荷": "electricLoads",
        "燃料": "fuels",
        "热": "HVACHeating",
        "冷": "HVACCooling",
        "常数电价": "常数电价",
        "分时电价": "分时电价",
        "阶梯电价": "阶梯电价",
        "分时阶梯电价": "分时阶梯电价",
        "台风灾害": "typhoon",
        "降雨灾害": "rainfall",
        "地震灾害": "earthquake",
        "极寒灾害": "extremeCold",
    }
    _kindUrlMap = {
        "PhotovoltaicSys": "api/ieslab-simulation/rest/dpcs/",
        "WindPowerGenerator": "api/ieslab-simulation/rest/dpcs/",
        "GasTurbine": "api/ieslab-simulation/rest/dpcs/",
        "HeatPump": "api/ieslab-simulation/rest/dhscs/",
        "GasBoiler": "api/ieslab-simulation/rest/dhscs/",
        "HPSolarCollector": "api/ieslab-simulation/rest/dhscs/",
        "CompRefrg": "api/ieslab-simulation/rest/dhscs/",
        "AbsorptionChiller": "api/ieslab-simulation/rest/dhscs/",
        "Battery": "api/ieslab-simulation/rest/escs/",
        "WaterTank": "api/ieslab-simulation/rest/escs/",
        "Transformer": "api/ieslab-simulation/rest/dstcs/",
        "TransferLine": "api/ieslab-simulation/rest/dstcs/",
        "MMC": "api/ieslab-simulation/rest/dstcs/",
        "CentrifugalPump": "api/ieslab-simulation/rest/hstcs/",
        "Pipe": "api/ieslab-simulation/rest/hstcs/",
        "thermalLoads": "api/ieslab-simulation/rest/thermalLoads/",
        "electricLoads": "api/ieslab-simulation/rest/electricLoads/",
        "fuels": "api/ieslab-simulation/rest/fuels/",
        "HVACHeating": "api/ieslab-simulation/rest/hots/",
        "HVACCooling": "api/ieslab-simulation/rest/colds/",
        "常数电价": "api/ieslab-simulation/rest/elects/",
        "分时电价": "api/ieslab-simulation/rest/elects/",
        "阶梯电价": "api/ieslab-simulation/rest/elects/",
        "分时阶梯电价": "api/ieslab-simulation/rest/elects/",
        "typhoon":  "api/ieslab-simulation/rest/typhoon/",
        "rainfall": "api/ieslab-simulation/rest/rainfall/",
        "earthquake": "api/ieslab-simulation/rest/earthquake/",
        "extremeCold": "api/ieslab-simulation/rest/extremeCold/",
    }
    pass


class IESPlanDataManageModel(DataManageModel):
    _baseUri = 'api/ieslab-plan/'
    _weatherUrl = 'api/ieslab-plan/rest/weather_data/'
    _kindNameMap = {
        "光伏": "PhotovoltaicSys",
        "风机": "WindPowerGenerator",
        "燃气轮机": "GasTurbine",
        "热泵": "HeatPump",
        "燃气锅炉": "GasBoiler",
        "热管式太阳能集热器": "HPSolarCollector",
        "单工况制冷机": "SingleConChiller",
        "双工况制冷机": "DualConChiller",
        "吸收式制冷机": "AbsorptionChiller",
        "蓄电池": "Battery",
        "储水罐": "WaterTank",
        "蓄冰空调": "IceStorageAC",
        "变压器": "Transformer",
        "传输线": "TransferLine",
        "模块化多电平变流器": "MMC",
        "离心泵": "CentrifugalPump",
        "管道": "Pipe",
        "采暖制冷负荷": "thermalLoads",
        "电负荷": "electricLoads",
        "燃料": "fuels",
        "热": "HVACHeating",
        "冷": "HVACCooling",
        "常数电价": "常数电价",
        "分时电价": "分时电价",
        "阶梯电价": "阶梯电价",
        "分时阶梯电价": "分时阶梯电价",
        "台风灾害": "typhoon",
        "降雨灾害": "rainfall",
        "地震灾害": "earthquake",
        "极寒灾害": "extremeCold",
        "PEM燃料电池": "PEMF",
        "SOFC": "SOFC",
        "碱性电解槽": "ALK",
        "PEM电解槽": "PEME",
        "SOEC": "SOEC",
        "储氢罐": "HydrogenTank",
        "储热罐": "HeatStoTank",
        "换热器": "HeatExchanger",
        "氢气压缩设备": "HydrogenCompression",
        "氢负荷": "HydrogenLoad",
        "化石燃料制氢": "HydrogenProductionFuels",
        "运氢槽车": "HydrogenTanker"
    }
    _kindUrlMap = {
        "PhotovoltaicSys": "api/ieslab-plan/rest/dpcs/",
        "WindPowerGenerator": "api/ieslab-plan/rest/dpcs/",
        "GasTurbine": "api/ieslab-plan/rest/dpcs/",
        "HeatPump": "api/ieslab-plan/rest/dhscs/",
        "GasBoiler": "api/ieslab-plan/rest/dhscs/",
        "HPSolarCollector": "api/ieslab-plan/rest/dhscs/",
        "SingleConChiller": "api/ieslab-plan/rest/dhscs/",
        "DualConChiller": "api/ieslab-plan/rest/dhscs/",
        "AbsorptionChiller": "api/ieslab-plan/rest/dhscs/",
        "Battery": "api/ieslab-plan/rest/escs/",
        "WaterTank": "api/ieslab-plan/rest/escs/",
        "IceStorageAC": "api/ieslab-plan/rest/escs/",
        "Transformer": "api/ieslab-plan/rest/dstcs/",
        "TransferLine": "api/ieslab-plan/rest/dstcs/",
        "MMC": "api/ieslab-plan/rest/dstcs/",
        "CentrifugalPump": "api/ieslab-plan/rest/hstcs/",
        "Pipe": "api/ieslab-plan/rest/hstcs/",
        "thermalLoads": "api/ieslab-plan/rest/thermalLoads/",
        "electricLoads": "api/ieslab-plan/rest/electricLoads/",
        "fuels": "api/ieslab-plan/rest/fuels/",
        "HVACHeating": "api/ieslab-plan/rest/hots/",
        "HVACCooling": "api/ieslab-plan/rest/colds/",
        "常数电价": "api/ieslab-plan/rest/elects/",
        "分时电价": "api/ieslab-plan/rest/elects/",
        "阶梯电价": "api/ieslab-plan/rest/elects/",
        "分时阶梯电价": "api/ieslab-plan/rest/elects/",
        "typhoon":  "api/ieslab-plan/rest/typhoon/",
        "rainfall": "api/ieslab-plan/rest/rainfall/",
        "earthquake": "api/ieslab-plan/rest/earthquake/",
        "extremeCold": "api/ieslab-plan/rest/extremeCold/",
        "HydrogenTank": "api/ieslab-plan/rest/escs/",
        "HeatStoTank": "api/ieslab-plan/rest/escs/",
        "HeatExchanger": "api/ieslab-plan/rest/hstcs/",
        "HydrogenCompression": "api/ieslab-plan/rest/hsec/",
        "HydrogenLoad": "api/ieslab-plan/rest/hydrogenLoad/",
        "HydrogenProductionFuels": "api/ieslab-plan/rest/hydrogen/",
        "HydrogenTanker": "api/ieslab-plan/rest/hydrogen/",
        "PEMF": "api/ieslab-plan/rest/dhscs/",
        "SOFC": "api/ieslab-plan/rest/dhscs/",
        "ALK": "api/ieslab-plan/rest/hpc/",
        "PEME": "api/ieslab-plan/rest/hpc/",
        "SOEC": "api/ieslab-plan/rest/hpc/",
    }
    pass


import StabilityAnalysis as SA
# import ReadSOT as rs

import cloudpss
import os
import time
import json
import re
import numpy as np
import numpy.linalg as LA

# %matplotlib
import matplotlib as mpl
import matplotlib.pyplot as plt
# import plotly.graph_objects as go
# from plotly.subplots import make_subplots
# import plotly.io as pio
# from scipy import interpolate

import pandas as pd

import tkinter
import tkinter.filedialog

import math

# from IPython.display import HTML
from html import unescape

import random
import json
import copy
from cloudpss.model.implements.component import Component
from cloudpss.runner.result import (Result, PowerFlowResult, EMTResult)
from cloudpss.model.revision import ModelRevision
from cloudpss.model.model import Model

# from docx import Document
# from docx.shared import Inches
# from docx.oxml.ns import qn
# from docx.shared import Pt,RGBColor
# from docx.oxml import OxmlElement

# from scipy import optimize
if __name__ == '__main__':
    tk = 'eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6NTk4LCJ1c2VybmFtZSI6Imx0aDIwMjEwMTIzNDkiLCJzY29wZXMiOltdLCJ0eXBlIjoiYXBwbHkiLCJleHAiOjE3MjEwNDA3MzYsImlhdCI6MTY4OTkzNjczNn0.kLc6IyB-725vTvj8wWJhWiFAvaDbET5wJUzUOP_SxZTlrBJB19Z-USjKaZW8P8COHqB37mxcMPenXhnhRqU4ZQ'
    apiURL = 'http://10.10.1.33:60002/'
    username = 'lth2021012349'
    projectKey = 'IEEE39_lth'
    cloudpss.setToken(tk)
    os.environ['CLOUDPSS_API_URL'] = apiURL

    # with open('saSource.json', "r", encoding='utf-8') as f:
    #     compLib = json.load(f)  # 添加cloupss平台所有元件库

    sa = SA.StabilityAnalysis()
    sa.setConfig(tk, apiURL, username, projectKey)
    sa.setInitialConditions()  # 初始化，调用cloudpss算例
    # sa.createSACanvas()  # 创造故障图层+暂态监控图层
    N_1LineKey1 = []

    ##  可以设置故障。详见以下函数：
    #     * setGroundFault(self, pinName, fault_start_time, fault_end_time,  fault_type, OtherParas = None) # 接地故障生成,pinName为接地故障引脚位置
    #     * setBreaker_3p(self, busName1, busName2, ctrlSigName = None, OtherParas = None) # busName1和busName2之间插入开关
    #     * setN_1(self, transKey, cut_time, OtherBreakerParas = None) # 设置传输线断路
    #     * setN_1_GroundFault(self, transKey,side, fault_start_time, cut_time, fault_type, OtherFaultParas = None, OtherBreakerParas = None) # N-1断线+接地故障生成
    #     * setN_2_GroundFault(self, transKey1, transKey2, side, fault_start_time, cut_time, fault_type, OtherFaultParas = None, OtherBreakerParas = None) # # N-2断线+接地故障生成

    # N_1LineLabel1 = 'TLine_3p-22'  # 传输线26-29
    # N_1LineKey1 = [j for j in sa.compLabelDict[N_1LineLabel1].keys()][0]
    # LineComp1 = sa.project.getComponentByKey(N_1LineKey1)

    # # 三相断路接地故障生成
    # # fault_type对应平台故障电阻故障类型
    # sa.setGroundFault(LineComp1.pins['0'], 3, 3.1, 7)

    # # 单次N-1断线+接地故障生成
    # sa.setN_1_GroundFault(N_1LineKey1, 0, 4, 4.06, 7)

    # 单次N-2断线+接地故障生成
    # N_2LineLabel1 = ['AC701004','AC701003']
    # for i in N_2LineLabel1:
    #     N_2LineKey1.append([j for j in sa.compLabelDict[i].keys()][0])
    # sa.setN_2_GroundFault(N_2LineKey1[0], N_2LineKey1[1], 0, 4, 4.09, 7)

    # 仿真任务设置
    jobName = 'SA_电磁暂态仿真(旧版本)'
    timeend = 10
    sa.createJob('emtp', name=jobName, args={'begin_time': 0, 'end_time': timeend, 'step_time': 0.00005, \
                                             #    'task_queue': 'taskManager_turbo1','solver_option': 7,'n_cpu': 16})
                                             'task_queue': 'taskManager_turbo1', 'solver_option': 7, 'n_cpu': 4})

    # 设置参数方案，无修改则默认参数方案
    sa.createConfig(name='SA_参数方案')

    # 设置输出
    # sa.addVoltageMeasures('SA_电磁暂态仿真', VMin = 220, freq = 200, PlotName = '220kV以上电压曲线') # 基于电压筛选
    sa.addOutputs('SA_电磁暂态仿真(旧版本)', {'0': 'Gen30,Gen38相对Gen39的功角', '1': 200, '2': 'compressed', '3': 1,
                                '4': ['canvas_11_1085', 'canvas_11_1083']})
    sa.runProject(jobName='SA_电磁暂态仿真(旧版本)', configName='SA_参数方案')

    # 输出通道曲线绘制
    sa.plotResult(sa.runner.result, 0)  # 绘制通道1

    # path = 'results/'+ projectKey
    # folder = os.path.exists(path)
    # if not folder:                   #判断是否存在文件夹如果不存在则创建为文件夹
    #     os.makedirs(path)            #makedirs 创建文件时如果路径不存在会创建这个路径

    # Result.dump(sa.runner.result,path+'/SAresult_'+ time.strftime("%Y_%m_%d_%H_%M_%S", time.localtime())+'.cjob')
    # SAresult = sa.runner.result

#     sa.displayPFResult()

##  修改cloudpss算例后另存修改后的算例
#     desc = '此为自动暂稳分析程序生成的仿真算例。\n作者：谭镇东；\n日期与时间：'+time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())+\
#             '\n' + sa.project.description
#     sa.saveProject(projectKey+'Auto_SA',sa.project.name+'_自动暂稳分析',desc = desc + sa.project.description )


import time
def foo():
    print("starting...")
    for i in range(5):
        print("before yield...",i)
        res = yield 4
        print("res:",res)
g = foo()
print(next(g))
print("*"*20)
time.sleep(2)
print(next(g))
print(next(g))
print(next(g))
print(next(g))
print(next(g))



import json

from cloudpss.ieslab.DataManageModel import IESPlanDataManageModel
from cloudpss.ieslab.EvaluationModel import IESLabEvaluationModel
from cloudpss.ieslab.PlanModel import IESLabPlanModel
from cloudpss.runner.IESLabTypicalDayResult import IESLabTypicalDayResult
from ..utils import request
from ..model.model import Model
from cloudpss.runner.runner import  Runner
from cloudpss.runner.IESLabPlanResult import IESLabPlanResult
from cloudpss.runner.IESLabEvaluationResult import IESLabEvaluationResult

class IESLabPlan(object):
    def __init__(self, project={}):
        '''
            初始化
        '''
        self.id = project.get('id', None)
        self.name = project.get('name', None)
        self.__modelRid = project.get('model', None)
        self.project_group = project.get('project_group', None)
        if self.__modelRid is not None:
            self.model = Model.fetch(self.__modelRid)
        self.dataManageModel = IESPlanDataManageModel(self.id)
        self.planModel = IESLabPlanModel(self.id)
        self.evaluationModel = IESLabEvaluationModel(self.id)
        self.currentPlanResult = IESLabPlanResult(self.id)
        self.currentEvaluationResult = IESLabEvaluationResult(self.id)

    @staticmethod
    def fetch(simulationId):
        '''
            获取算例信息

            :params: simulationId string类型，代表数据项的算例id

            :return: IESLabPlan
        '''
        try:
            r = request('GET',
                        'api/ieslab-plan/rest/simu/{0}/'.format(simulationId))
            project = json.loads(r.text)
            return IESLabPlan(project)
        except:
            raise Exception('未查询到当前算例')

    def __run(self, job=None, name=None):
        '''
            调用仿真 

            :params job:  调用仿真时使用的计算方案，不指定将使用算例保存时选中的计算方案
            :params name:  任务名称，为空时使用项目的参数方案名称和计算方案名称

            :return: 返回一个运行实例
        '''
        if job is None:
            currentJob = self.model.context['currentJob']
            job = self.model.jobs[currentJob]
        job['args']['simulationId'] = self.id
        return self.model.run(job, name=name)

    def iesLabTypicalDayRun(self, job=None, name=None, **kwargs)->Runner[IESLabTypicalDayResult]:
        '''
            运行典型日计算 

            :params job:  调用仿真时使用的计算方案，不指定将使用算例保存时选中的计算方案
            :params name:  任务名称，为空时使用项目的参数方案名称和计算方案名称

            :return: Runner[IESLabTypicalDayResult]
        '''
        if job is None:
            currentJob = self.model.context['currentJob']
            job = self.model.jobs[currentJob]
            if job['rid'] != 'function/CloudPSS/ieslab-gmm':
                for j in self.model.jobs:
                    if j['rid'] == 'job-definition/ies/ies-gmm' or j['rid'] == 'job-definition/cloudpss/ieslab-gmm':
                        j['rid'] = 'function/CloudPSS/ieslab-gmm'
                        job = j
        if job is None:
            raise Exception("找不到默认的综合能源系统规划典型日生成算法的计算方案")
        if job['rid'] != 'function/CloudPSS/ieslab-gmm':
            raise Exception("不是综合能源系统规划典型日生成算法的计算方案")
        return self.__run(job=job, name=name)

    def iesLabEvaluationRun(self, planId, type=None):
        '''
            运行方案评估

            :param planID int类型，表示优化方案的ID，数值位于0~优化方案数量之间
            :param type string类型，表示评估类型，可选值为：能效评价、环保评价

            :return: 方案评估运行实例

        '''
        return self.evaluationModel.run(planId, type)

    def iesLabEnergyEvaluationRun(self, planId):
        '''
            运行能效评价

            :param planID int类型，表示优化方案的ID，数值位于0~优化方案数量之间

            :return: 能效评价运行实例

        '''
        return self.evaluationModel.EnergyEvaluationRun(planId)

    def iesLabEnvironmentalEvaluationRun(self, planId):
        '''
            运行环保评价

            :param planID int类型，表示优化方案的ID，数值位于0~优化方案数量之间

            :return: 环保评价运行实例
        '''
        return self.evaluationModel.EnvironmentalEvaluationRun(planId)

    def iesLabPlanRun(self):
        '''
            生成方案优选算例

            :return: 方案优选运行实例
        '''
        return self.planModel.run()
    
    def iesLabPlanKill(self):
        '''
            停止并删除方案优选算例

            :return: Boolean
        '''
        return self.planModel.kill()
    
    def createProjectGroup(self, name):
        '''
            创建项目组

            :params name: 项目组名称

            :return: 返回一个创建的项目组
        '''

    @staticmethod
    def createProjectGroup(group_name, desc=None, createById=None):
        '''
            创建项目组

            :params group_name: String 项目组名称 
            :params desc: String 项目组描述 可选参数
            :params createById Int 父项目组id  可选参数，如果是从已有项目组导入的项目组，必填此项

            :return: Int 返回创建的项目组id
        '''
        try:
            if createById is None: 
                isImport = 0
            else:
                isImport = 1
            payload = {
                'group_name': group_name,
                'desc': desc,
                'isImport': isImport,
                'createById': createById,
            }
            r = request(
                'POST', 'api/ieslab-plan/rest/projectgroup/', data=json.dumps(payload))
            project = json.loads(r.text)
            return project.get('id', None)
        except Exception as e:
            raise Exception('创建项目组失败')

    @staticmethod  
    def createProject(name, project_group, start_date, end_date, construction_cycle, desc=None, createById=None):
        '''
            创建项目

            :params name: String 项目名称 
            :params project_group: Int 父项目组id
            :param start_date: Int 项目开始年限，范围在[1500,3000]之间
            :param end_date: Int 项目结束年限，范围在项目开始时间之后且不超过五十年
            :param construction_cycle: Int 项目建设周期(年), 必须小于等于 项目结束年限 - 项目开始年限
            :params desc: String 项目描述, 可选参数
            :params createById Int 父项目id, 可选参数, 如果是从已有项目导入的项目，必填此项

            :return: Int 返回创建的项目id
        '''
        try:
            if start_date < 1500 or start_date > 3000:
                raise Exception('项目开始年限错误，范围在[1500,3000]之间')
            if end_date < start_date or end_date > start_date + 50:
                raise Exception('项目结束年限错误，范围在项目开始时间之后且不超过五十年')
            if construction_cycle > end_date - start_date:
                raise Exception('项目建设周期错误，必须小于等于 项目结束年限 - 项目开始年限')
            if createById is None: 
                payload = {
                    'name': name,
                    'project_group': project_group,
                    'start_date': start_date,
                    'end_date': end_date,
                    'construction_cycle': construction_cycle,
                    'desc': desc
                }
            else:
                payload = {
                    'name': name,
                    'project_group': project_group,
                    'start_date': start_date,
                    'end_date': end_date,
                    'construction_cycle': construction_cycle,
                    'desc': desc,
                    'createById': createById
                }
            r = request(
                'POST', 'api/ieslab-plan/rest/simu/', data=json.dumps(payload))
            project = json.loads(r.text)
            return project.get('id', None)
        except Exception as e:
            raise Exception('创建项目失败')




import re
import os
import io
import datetime
import time
COLOR_LIST = [
    'var(--spectrum-global-color-celery-700)',
    'var(--spectrum-global-color-chartreuse-700)',
    'var(--spectrum-global-color-yellow-700)',
    'var(--spectrum-global-color-magenta-700)',
    'var(--spectrum-global-color-fuchsia-700)',
    'var(--spectrum-global-color-purple-700)',
    'var(--spectrum-global-color-indigo-700)',
    'var(--spectrum-global-color-seafoam-700)',
    'var(--spectrum-global-color-red-700)',
    'var(--spectrum-global-color-orange-700)',
    'var(--spectrum-global-color-green-700)',
    'var(--spectrum-global-color-blue-700)',
    'var(--spectrum-global-color-celery-400)',
    'var(--spectrum-global-color-chartreuse-400)',
    'var(--spectrum-global-color-yellow-400)',
    'var(--spectrum-global-color-magenta-400)',
    'var(--spectrum-global-color-fuchsia-400)',
    'var(--spectrum-global-color-purple-400)',
    'var(--spectrum-global-color-indigo-400)',
    'var(--spectrum-global-color-seafoam-400)',
    'var(--spectrum-global-color-red-400)',
    'var(--spectrum-global-color-orange-400)',
    'var(--spectrum-global-color-green-400)',
    'var(--spectrum-global-color-blue-400)',
    'var(--spectrum-global-color-celery-500)',
    'var(--spectrum-global-color-chartreuse-500)',
    'var(--spectrum-global-color-yellow-500)',
    'var(--spectrum-global-color-magenta-500)',
    'var(--spectrum-global-color-fuchsia-500)',
    'var(--spectrum-global-color-purple-500)',
    'var(--spectrum-global-color-indigo-500)',
    'var(--spectrum-global-color-seafoam-500)',
    'var(--spectrum-global-color-red-500)',
    'var(--spectrum-global-color-orange-500)',
    'var(--spectrum-global-color-green-500)',
    'var(--spectrum-global-color-blue-500)',
    'var(--spectrum-global-color-celery-600)',
    'var(--spectrum-global-color-chartreuse-600)',
    'var(--spectrum-global-color-yellow-600)',
    'var(--spectrum-global-color-magenta-600)',
    'var(--spectrum-global-color-fuchsia-600)',
    'var(--spectrum-global-color-purple-600)',
    'var(--spectrum-global-color-indigo-600)',
    'var(--spectrum-global-color-seafoam-600)',
    'var(--spectrum-global-color-red-600)',
    'var(--spectrum-global-color-orange-600)',
    'var(--spectrum-global-color-green-600)',
    'var(--spectrum-global-color-blue-600)',
];

class Transformer():

    def __init__(self, job):
        self.drawMetadata = None
        self.job = job
        self.firstDraw = True

    def __msg(self, data, level):
        rex = r'^\[\s*\d{1,4}\/\d{1,2}\/\d{1,4}\s+\d{2}:\d{2}:\d{2}\s*\]\s*\[(critical|warning|error|info|verbose|debug)\]\s*'
        match = re.match(rex, data, re.I | re.S)
        if match:
            data = data[len(match.group(0)):]
            level = match.group(1)
        if re.match(r'^\s*run ends!?\s*$', data, re.I | re.S):
            return {
                'type': 'terminate',
                'version': 1,
                'key': '%.9f' % time.time(),
                'data': {
                    'succeeded':
                        'resolved' if self.previousLevel != 'critical' and
                        self.previousLevel != 'error' and
                        level != 'critical' and
                        level != 'error' else 'rejected',
                },
                # 'id':id,
                'sender': 'remote',
                'when': datetime.datetime.now(),
            }
        self.previousLevel = level
        return {
            'type': 'log',
            'verb': 'create',
            'version': 1,
            'key': '%.9f' % time.time(),
            'data': {
                'level': level,
                'content': data,
            },
            # 'id':id,
            'sender': 'remote',
            'when': datetime.datetime.now()
        }

    def __draw(self, data):
        globalBegin = self.job['args']['begin_time']
        globalEnd = self.job['args']['end_time']
        plots = []
        if data is None:
            return None
        if not self.drawMetadata:

            resultData = []
            plot = {'plot-1': {
                'type': 'plot',
                'key': 'plot',
                'verb': 'create' if self.firstDraw else 'append',
                'version': 1,
                'data': {
                    'traces': resultData,
                },
                'sender': 'remote',
                'when': datetime.datetime.now()
            }}
            for key, val in data.items():
                x, y = zip(*val)
                resultData.append({
                    'name': key,
                    'type': 'scatter',
                    'x': list(x),
                    'y': list(y),
                })
            self.firstDraw = False
            return plot

        else:
            for i in range(len(self.drawMetadata)):
                plotKey = 'plot-{0}'.format(i)
                meta = self.drawMetadata[i]

                axis = None
                try:
                    if meta['type'] == 'oscilloscope':
                        if meta.get('width', None) is not None:
                            dataSample = data[meta['traces'][0]['key']]
                            if dataSample is not None:
                                end = dataSample[len(dataSample)-1][0]
                                remaining = end % float(meta['width'])
                                if remaining <= 0.000001:
                                    remaining = meta['width']
                                axis = {
                                    'range': [end - remaining, end - remaining + meta['width']]}

                    elif meta['type'] == 'moving':
                        if meta.get('width', None) is not None:
                            begin = globalBegin
                            dataSample = data[meta['traces'][0]['key']]
                            end = dataSample[len(dataSample) - 1][0]
                            begin = max(
                                float(end)-float(meta['width']), float(begin))
                            axis = {'range': [begin, end]}

                    elif meta['type'] == 'global':
                        axis = {'range': [globalBegin, globalEnd]}
                except Exception as e:
                    print(e)
                resultData = []
                plot = {
                    'type': 'plot',
                    'key': plotKey,
                    'verb': 'create' if self.firstDraw else 'append',
                    'version': 1,
                    'data': {
                        'title': meta['title'] if self.firstDraw else None,
                        'xAxis': axis,
                        'traces': resultData,
                    },
                    'sender': 'remote',
                    'when': datetime.datetime.now()
                }
                traces = meta['traces']
                for val in traces:
                    element = data.get(val['key'], None)
                    if not element:
                        resultData.append({
                            'name': val['name'],
                            'type': 'scatter',
                            'x': [],
                            'y': [],
                        })
                        continue
                    x, y = zip(*element)
                    resultData.append({
                        'name': val['name'],
                        'type': 'scatter',
                        'x': list(x),
                        'y': list(y),
                    })
                plots.append(plot)
            self.firstDraw = False
            return plots

    def __compValues(self, data, field):
        if data is None:
            return None
        components = {}
        for key, val in data.items():
            if not key.startswith('/'):
                continue
            compKey = key[1:]
            components[compKey] = {field: val}
        result = {
            'type': 'modify',
            'version': 1,
            'data': {
                'payload': {
                    'revision': {
                        'implements': {
                            'diagram': {
                                'components': components,
                            }
                        },
                    },
                },
            }
        }
        if self.job['rid'] == 'job-definition/cloudpss/power-flow':
            if field == 'args':
                result['data']['title'] = '潮流初始化断面'
                result['data']['content'] = '将潮流结果写回模型作为暂态仿真初始化断面'
            elif field == 'context':
                result['data']['title'] = '潮流可视化展示'
                result['data']['content'] = '在图纸上展示此潮流结果'
        return result

    def __status(self, data, field):
        if data is None:
            return None
        cells = {}
        for key, val in data.items():
            if not key.startswith('/'):
                continue
            compKey = key[1:]

            cells[compKey] = {field: {
                '--stroke': COLOR_LIST[int(val['color']) % len(COLOR_LIST)], 'title': val['value']}}
        result = {
            'type': 'modify',
            'version': 1,
            'data': {
                'payload': {
                    'revision': {
                        'implements': {
                            'diagram': {
                                'cells': cells,
                            }
                        },
                    },
                },
            }
        }
        if self.job['rid'] == 'job-definition/cloudpss/emtp':
            result['data']['title'] = '分网结果'
            result['data']['content'] = '在拓扑中显示分网结果'
        return result

    def transform(self, message):

        if message['cmd'] == 'msg':
            return self.__msg(message['data'], 'info')

        elif message['cmd'] == 'errormsg':
            return self.__msg(message['data'], 'error')

        elif message['cmd'] == 'terminate':
            data = message['data']
            return {
                'type': 'terminate',
                'version': 1,
                'data': {'succeeded': data.get('succeeded', True)},
            }
        elif message['cmd'] == 'message':
            data = message.data
            return {
                'type': 'log',
                'verb': 'create',
                'version': 1,
                'data': {
                    'level': data['level'],
                    'content': data['content'],
                    'html': True,
                },
                'key': '%.9f' % time.time(),
                'when': datetime.datetime.now(),

            }
        elif message['cmd'] == 'drawmeta':
            self.drawMetadata = message['data']

        elif message['cmd'] == 'draw' or message['cmd'] == 'chart':
            data = message.get('data', None)
            return self.__draw(data)

        elif message['cmd'] == 'compParams':
            data = message.get('data', None)
            if data is None:
                return None
            app = data.get('defaultApp', None)
            msg = self.__compValues(app, 'args')
            return msg
        elif message['cmd'] == 'context':
            data = message.get('data', None)
            return self.__compValues(data, 'context')
        elif message['cmd'] == 'status':
            data = message.get('data', None)
            return self.__status(data, 'style')
        return None



import asyncio


async def consumer(n, q):
    print('consumer {}: starting'.format(n))
    while True:
        print('consumer {}: waiting for item'.format(n))
        item = await q.get()
        print('consumer {}: has item {}'.format(n, item))
        if item is None:
            # None 是一个停止信号。
            q.task_done()
            break
        else:
            await asyncio.sleep(n)
            q.task_done()
    print('consumer {}: ending'.format(n))


async def producer(q, num_workers):
    print('producer: starting')
    # 向队列中添加一些数字来模拟作业
    for i in range(num_workers * 30):
        await q.put(i)
        await asyncio.sleep(0.1)
        print('生产者: 添加任务 {} 到队列'.format(i))
        # 添加 None 到队列
    # 发出消费者退出的信号
    print('生产者: 向队列添加停止信号')
    for i in range(num_workers):
        await q.put(None)
    print('producer: waiting for queue to empty')
    await q.join()
    print('producer: ending')


async def main(loop, num_consumers):
    # 创建具有固定大小的队列，用于生产者
    # 将阻塞，直到消费者取出一些item。
    q = asyncio.Queue(maxsize=num_consumers)

    # 消费者任务调度
    consumers = [
        loop.create_task(consumer(i, q))
        for i in range(num_consumers)
    ]

    # 生产者任务调度
    prod = loop.create_task(producer(q, num_consumers))

    # 等待所有协程完成。
    await asyncio.wait(consumers + [prod])


event_loop = asyncio.get_event_loop()
try:
    event_loop.run_until_complete(main(event_loop, 2))
finally:
    event_loop.close()



# coding=UTF-8
import json
from cloudpss.utils import request


def graphql_request(query, variables=None):
    payload = {'query': query, 'variables': variables}
    r = request('POST', 'graphql', data=json.dumps(payload))

    return json.loads(r.text)



import os, sys

sys.path.append(os.path.join(os.path.dirname(__file__), '../'))
import cloudpss
import json
import time
if __name__ == '__main__':
    print('start')
    os.environ['CLOUDPSS_API_URL'] = 'http://10.101.10.45/'
    cloudpss.setToken('eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6MSwidXNlcm5hbWUiOiJhZG1pbiIsInNjb3BlcyI6WyJtb2RlbDo5ODMzNSIsImZ1bmN0aW9uOjk4MzM1IiwiYXBwbGljYXRpb246OTgzMzUiXSwicm9sZXMiOlsiYWRtaW4iXSwidHlwZSI6ImFwcGx5IiwiZXhwIjoxNzI2MjA4NjM5LCJub3RlIjoidGVzdDIiLCJpYXQiOjE2OTUxMDQ2Mzl9.ZlFSPCH3u4OSBlDBJWve4GZkWIZAku_DI1j-or-uDtAtyDy-RZecn7RBhymsILNPsmKnwkXcx1UnsjRNiAf5yA')
    ts1 = time.time()
    ### 获取指定 rid 的项目
    project = cloudpss.Model.fetch('model/CloudPSS/IEEE3')

    ts2 = time.time()
    print('time 1:', ts2 - ts1, flush=True)
    runner = project.run()
    while not runner.status():
        # print('running', flush=True)
        logs = runner.result.getLogs()
        if (len(logs) > 1):
            print(logs)
            t = time.time()
            print('s time 2:', t - ts1, t - ts2, flush=True)
        # for log in logs:
        #     print(log)
        time.sleep(0.1)

    # 打印 0 号分组数据
    # print(runner.result.getPlot(0))
    # filePath = 'D:\\data\\result\\test.cjob'
    # cloudpss.Result.dump(runner.result, filePath)
    print('end')



import copy
import re
import collections

from cloudpss.utils.httprequests import request
from .storage import Storage

class Result(object):
    """
        结果处理类，从消息存储库中获取数据，并进行简单的整理

        可迭代器，迭代时按接收顺序返回数据

        >>> for data in result:
                print(data)

        也可以从类的 db 变量，获取数据存储类实例进行操作
    """
    def __init__(self, db):
        """
            初始化
        """
        self.n = 0
        self.db = db
        self.__logsIndex = 0

    @classmethod
    def load(cls, filePath):
        """
            加载本地结果文件

            :params: file 文件目录

            :return: 返回一个结果实例

            >>> result = Result.load('C:\\Users\\dps-dm\\cloudpss-sdk\\result\\424111.cjob')

        """
        db = Storage.load(filePath)

        return cls(db)

    @staticmethod
    def dump(result, file):
        '''
            保存结果到本地文件

            :params: file 保存文件的目录

            >>> Result.dump(file)
            {...}
        '''

        Storage.dump(result.db, file)

    def __iter__(self):
        return self

    def __next__(self):
        maxLength = self.db.getMessageLength()
        if self.n < maxLength:
            message = self.db.getMessage(self.n)
            self.n += 1
            return message
        raise StopIteration()

    def getLogs(self):
        '''
            获取当前任务的日志

            >>>logs= result.getLogs()
            {...}
        '''
        result = []
        length = self.db.getMessageLength()
        if (length > self.__logsIndex):
            for num in range(self.__logsIndex, length):
                val = self.db.getMessage(num)
                if val['type'] == 'log':
                    result.append(val)
            self.__logsIndex = length
        return result

    def __deepModify(self, dict1, dict2):

        for key, val in dict1.items():
            if type(val) is dict:
                self.__deepModify(val, dict2[key])
            else:
                dict2[key] = val

    def modify(self, data, model):
        """
            通过指定消息修改算例文件

            :params: data 消息字典 {}
            :params: model  项目

            >>> message= result.modify(data,model)

        """
        modifyData = data['data']
        payload = modifyData['payload']
        self.__deepModify(payload, model)

    def getMessagesByType(self, type):
        """
            获取指定类型的消息数据

            

            >>> message= result.getMessagesByType('log')
        """

        return self.db.getMessagesByType(type)


class EMTResult(Result):
    """
        电磁暂态结果处理类，继承 Result， 

        提供快捷 plot 数据的接口函数，获取到的 plot 数据为合并后的数据格式，不在是接收时分段的数据

        该类只提供 EMT 仿真使用

    """
    def __init__(self, *args, **kwargs):
        """
            初始化
        """
        Result.__init__(self, *args, **kwargs)
        self.result = {}
        self.__plotIndex = 0

    def getPlots(self):
        '''
            获取所有的 plots 数据

            >>> result.getPlots()
            {...}
        '''

        length = self.db.getMessageLength()
        if (length > self.__plotIndex):
            for num in range(self.__plotIndex, length):
                val = self.db.getMessage(num)
                if val['type'] == 'plot':
                    key = val['key']
                    if self.result.get(key, None) is None:
                        self.result[key] = copy.deepcopy(val)
                    else:
                        traces = val['data']['traces']
                        for i in range(len(traces)):
                            v = traces[i]
                            self.result[key]['data']['traces'][i]['x'].extend(
                                v['x'])
                            self.result[key]['data']['traces'][i]['y'].extend(
                                v['y'])
            self.__plotIndex = length
        return self.result.values()

    def getPlot(self, index: int):
        '''
            获取指定序号的数据分组

            :params: index 图表位置

            >>> result.getPlot(0)
            {...}
        '''
        self.getPlots()
        if self.result is not None:
            return self.result.get('plot-{0}'.format(int(index)), None)

    def getPlotChannelNames(self, index):
        '''
            获取一组输出分组下的所有通道名称

            :params: index 输出通道位置

            :return: 通道名称数组

            >>>names= result.getPlotChannelNames(0)
            []
        '''
        plot = self.getPlot(int(index))
        if plot is None:
            return None

        return [val['name'] for val in plot['data']['traces']]

    def getPlotChannelData(self, index, channelName):
        '''
            获取一组输出分组下指定通道名称的数据

            :params: index 输出通道位置
            :params: channelName 输出通道名称

            :return: 通道数据, 一个 trace 数据

            >>>channel= result.getPlotChannelData(0，'')
            {...}
        '''
        plot = self.getPlot(int(index))
        if plot is None:
            return None
        for val in plot['data']['traces']:
            if val['name'] == channelName:
                return val
        return val


class PowerFlowResult(Result):
    """
        潮流结果处理类，继承 Result， 

        提供快速获取 buses 和 branches 的接口，并提供潮流写入项目的接口

        该类只提供潮流仿真时使用

    """
    def __init__(self, *args, **kwargs):
        """
            初始化
        """
        Result.__init__(self, *args, **kwargs)

    def getBuses(self):
        """
            获取潮流结果 buses 数据

            >>> channel= result.getBuses()
            [...]
        """

        result = []
        pattern = re.compile("value=\"(.*)\"")
        data = self.db.getMessagesByKey('buses-table')
        if len(data) == 0:
            return []
        table = copy.deepcopy(data[0])
        columns = table['data']['columns']
        for column in columns:
            if column['type'] == 'html':
                data = column['data']
                for i in range(len(data)):
                    if data[i] != '':
                        s = pattern.findall(data[i])
                        data[i] = s[0].replace('/', '')

        result.append(table)
        return result

    def getBranches(self):
        """
            获取潮流结果 branches 数据

            >>> channel= result.getBranches()
            [...]
        """

        result = []
        pattern = re.compile("value=\"(.*)\"")
        data = self.db.getMessagesByKey('branches-table')
        if len(data) == 0:
            return []
        table = copy.deepcopy(data[0])

        columns = table['data']['columns']
        for column in columns:
            if column['type'] == 'html':
                data = column['data']
                for i in range(len(data)):
                    if data[i] != '':
                        s = pattern.findall(data[i])
                        data[i] = s[0].replace('/', '')

        result.append(table)
        return result

    def powerFlowModify(self, model):
        """
            潮流数据写入 model

            >>> channel= result.powerFlowModify(model)
        """

        data = self.db.getMessagesByKey('power-flow-modify')
        if len(data) == 0:
            raise Exception('未找到到数据')
        self.modify(data[0], model)


class IESResult(Result):
    """
        综合能源结果处理类，继承 Result， 

        提供快捷 plot 数据的接口函数，获取到的 plot 数据为合并后的数据格式，不在是接收时分段的数据

        该类只提供 IES 仿真使用

    """
    def __init__(self, *args, **kwargs):
        """
            初始化
        """
        Result.__init__(self, *args, **kwargs)
        self.result = {'Sankey': []}
        self.__plotIndex = 0

    def __readPlotResult(self):
        length = self.db.getMessageLength()
        if (length > self.__plotIndex):
            for num in range(self.__plotIndex, length):
                val = self.db.getMessage(num)
                if val['type'] == 'plot':
                    key = val['key']
                    if key == 'Sankey':
                        self.result['Sankey'].append(copy.deepcopy(val))
                    else:
                        if self.result.get(key, None) is None:
                            self.result[key] = copy.deepcopy(val)
                        else:
                            traces = val['data']['traces']
                            for i in range(len(traces)):
                                v = traces[i]
                                self.result[key]['data']['traces'][i][
                                    'x'].extend(v['x'])
                                self.result[key]['data']['traces'][i][
                                    'y'].extend(v['y'])
            self.__plotIndex = length

    def getPlotData(self, compID, labelName, traceName='all', index=-1):
        '''
            获取元件ID为compID的元件，对应标签为labelName、图例名称为traceName的plot 数据的第index项

            :params: compID string类型，代表元件的标识符
            :params: labelName string类型，代表plot曲线的分组标签
            :params: traceName string类型，代表Plot曲线对应分组下的图例名称，当为'all'时，返回所有图例的数据
            :params: index int类型，代表对应图例时序数据中的第index项，当小于0时，返回该图例所有的时序数据
            
            :return: dict类型
        '''
        self.__readPlotResult()
        key = compID + '_' + labelName
        if key not in self.result.keys():
            raise Exception('未找到元件标志为{0},对应label为{1}的plot数据'.format(
                compID, labelName))

        traceData = self.result[key]['data']['traces']
        if traceName != 'all':
            traceNameList = [traceName]
        else:
            traceNameList = [
                traceData[i]['name'] for i in range(len(traceData))
            ]

        startIndex = 0
        endIndex = len(traceData[0]['x'])
        if index >= 0:
            startIndex = index
            endIndex = index + 1

        plotData = collections.defaultdict(lambda: {})
        for tName in traceNameList:
            for i in range(len(traceData)):
                dataLen = len(traceData[i]['x'])
                if traceData[i]['name'] == tName:
                    if endIndex > dataLen:
                        raise Exception('请求的index超过了plot数据序列的长度')
                    plotData[tName]['x'] = traceData[i]['x'][
                        startIndex:endIndex]
                    plotData[tName]['y'] = traceData[i]['y'][
                        startIndex:endIndex]

        return plotData

    def getSankey(self, index):
        '''
            获取第index个桑基图数据

            >>> result.getSankey(index)
            {...}
        '''
        self.__readPlotResult()
        if index >= len(self.result['Sankey']):
            raise Exception('index超过了桑基图数据序列的长度')
        return self.result['Sankey'][index]

    def getSankeyNum(self):
        '''
            获取桑基图数据序列的长度

            >>> result.getSankeyNum()
        '''
        self.__readPlotResult()
        return len(self.result['Sankey'])

class IESLabSimulationResult(IESResult):
    pass

class IESLabTypicalDayResult(IESResult):
    def GetTypicalDayNum():
        '''
            获取当前result的典型日数量
            
            :return: int类型，代表典型日数量
        '''
    def GetTypicalDayInfo(dayID):
        '''
            获取dayID对应典型日的基础信息
            
            :params: dayID int类型，表示典型日的ID，数值位于 0~典型日数量 之间
            
            :return: dict类型，代表典型日的基础信息，包括典型日所代表的日期范围、典型日的名称等
        '''
    def GetTypicalDayCurve(dayID, dataType):
        '''
            获取dayID对应典型日下dataType参数的时序曲线
            
            :params: dayID int类型，表示典型日的ID，数值位于 0~典型日数量 之间
            :params: dataType enum类型，标识辐照强度、环境温度、土壤温度、建筑物高度风速、风机高度风速、电负荷、热负荷、冷负荷的参数类型
            
            :return: list<float>类型，代表以1h为时间间隔的该参数的日内时序曲线
        '''
    pass



import sys
import os
sys.path.append(os.path.join(os.path.dirname(__file__), '..\\'))
import cloudpss
import time
import numpy as np
import pandas as pd
import json

if __name__ == '__main__':
    os.environ['CLOUDPSS_API_URL'] = 'http://10.101.10.45/'
    print('CLOUDPSS connected')
    cloudpss.setToken(
        'eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6MSwidXNlcm5hbWUiOiJhZG1pbiIsInNjb3BlcyI6WyJtb2RlbDo5ODMzNSIsImZ1bmN0aW9uOjk4MzM1IiwiYXBwbGljYXRpb246OTgzMzUiXSwicm9sZXMiOlsiYWRtaW4iXSwidHlwZSI6ImFwcGx5IiwiZXhwIjoxNzI0NTU3MDIzLCJub3RlIjoiYSIsImlhdCI6MTY5MzQ1MzAyM30._Xuyo62ESKLcIAFeNdnfBM44yPiiXli9OPKvXDzL2rPV4J1_qsGZP--bsS1tXAVy-x8ooUIIAAG1yhwmZuk7-Q')
    print('Token done')
    # project = cloudpss.Model.fetch('model/admin/7744b02b-0636-5a39-8c16-eca939259ee1')
    t = time.time()
    topology = cloudpss.ModelTopology.fetch("24sne1BQ_lLCRLBnVl7_9QdhR70HxKIWsmIpKMF4iBw6tTt-h6ZkB-vUgm-nfDp2","emtp",{'args':{}})
    # topology = cloudpss.ModelTopology.fetch("JwHbZdjco9eC0nZw3fY7Iz9rqsQ4HFGJObiBW3bFuYLPCd0Vqb2vb8zNY28D1AXV","emtp",{'args':{}})
    print(time.time()-t)
    
    
    
    # topology= project.fetchTopology(config={'args':{}})

    # topology.dump(topology,'test.json')
    
    
    


import os, sys

sys.path.append(os.path.join(os.path.dirname(__file__), '../'))
import cloudpss
import json
import time
if __name__ == '__main__':
    print('start')
    cloudpss.setToken(
        'eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6MSwidXNlcm5hbWUiOiJhZG1pbiIsInNjb3BlcyI6W10sInR5cGUiOiJhcHBseSIsImV4cCI6MTcyNjkwMjA0MCwiaWF0IjoxNjk1Nzk4MDQwfQ.qktX85dOW21jJb-sunrXZjiYSAj94ZRfh6XPE_pnMGsBMXffRc5zp40sDuJ4jVSlI2ak6ybAsm9vbY0cPWpiBg'
    )

    os.environ['CLOUDPSS_API_URL'] = 'http://10.101.10.119/'
    ts1 = time.time()
    ### 获取指定 rid 的项目
    model = cloudpss.Model.fetch('model/admin/core_test')

    job=model.jobs[2]
    job['args']['n_cpu']='8'
    job['args']['@debug']='min_cpu=0'
    
    ts2 = time.time()
    print('time 1:', ts2 - ts1, flush=True)
    runner = model.run()
    while not runner.status():
        time.sleep(1)

    logs = runner.result.getLogs()
    for log in logs:
        if log['type'] == 'log':
            if log['data']['content'].find('thread 0 total time')>0:
                print(log['data']['content'])
                break
    print('end')



import os

import json
import signal
import sys
from ..utils import MatlabDataEncoder


class EventSample(object):
    def __init__(self):
        self.__eventHandlerSample = []

    def __iadd__(self, EventHandler):
        self.__eventHandlerSample.append(EventHandler)
        return self

    def __isub__(self, EventHandler):
        self.__eventHandlerSample.remove(EventHandler)
        return self

    def __call__(self, *args, **kwargs):
        for [eventHandlerSample, eventHandlerArgs,
             eventHandlerKwargs] in self.__eventHandlerSample:
            eventHandlerSample(*eventHandlerArgs, **eventHandlerKwargs)


class Args(dict):
    """
        参数类
    """
    def __getattr__(self, key):
        """__getattr__
        """
        return self[key]


class FunctionExecution(object):

    _current = None
    """ 
        FunctionExecution
    """
    def _on_abort(self, frame, sigNum):
        if not self._aborted:
            self._aborted = True
            self.__abortedEvents()

    @staticmethod
    def current():
        """
        获取表示当前执行的 FunctionExecution 单例
        """
        jobId = os.environ.get('CLOUDPSS_JOB_ID', None)
        functionId = os.environ.get('CLOUDPSS_FUNCTION_ID', None)
        executorId = os.environ.get('CLOUDPSS_EXECUTOR_ID', None)
        executorName = os.environ.get('CLOUDPSS_EXECUTOR_NAME', None)
        executorVersion = os.environ.get('CLOUDPSS_EXECUTOR_VERSION', None)
        apiUrl = os.environ.get('CLOUDPSS_API_URL', None)
        gqlUrl = os.environ.get('CLOUDPSS_GQL_URL', None)
        homeUrl = os.environ.get('CLOUDPSS_HOME_URL', None)
        jobToken = os.environ.get('CLOUDPSS_JOB_TOKEN', None)
        functionToken = os.environ.get('CLOUDPSS_FUNCTION_TOKEN', None)
        current = FunctionExecution(id=jobId,
                                    functionId=functionId,
                                    executorId=executorId,
                                    executorName=executorName,
                                    executorVersion=executorVersion,
                                    apiUrl=apiUrl,
                                    gqlUrl=gqlUrl,
                                    homeUrl=homeUrl,
                                    token=jobToken,
                                    functionToken=functionToken)
        args = FunctionExecution.__loadArgs()
        current.args = args
        FunctionExecution._current = current
        return current

    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.functionId = kwargs.get('functionId', None)
        self.executorId = kwargs.get('executorId', None)
        self.executorName = kwargs.get('executorName', None)
        self.executorVersion = kwargs.get('executorVersion', None)
        self.apiUrl = kwargs.get('apiUrl', None)
        self.gqlUrl = kwargs.get('gqlUrl', None)
        self.homeUrl = kwargs.get('homeUrl', None)
        self.token = kwargs.get('token', None)
        self.functionToken = kwargs.get('functionToken', None)
        self.__abortedEvents = EventSample()
        self._args = None
        signal.signal(signal.SIGINT, self._on_abort)

    @property
    def args(self):
        return self._args

    @args.setter
    def args(self, val):
        self._args = val

    @staticmethod
    def __loadArgs():
        """加载当前任务参数

        Returns:
            dict: 任务参数
        """
        args = Args()
        for k, v in os.environ.items():
            if k.startswith('CLOUDPSS_JOB_ARG-'):
                s = k.split('-')
                key = bytes.decode(bytes.fromhex(s[1]))
                # print(key, v, flush=True)

                args[key] = json.loads(v)
        return args

    def feedDog(self):
        """
            通知看门狗，当前程序还在运行，当程序 30s 内没有输出时，执行器将直接 kill 运行脚本
        """
        print("", flush=True)

    _aborted = False
    '''
    是否已取消当前任务

    当任务被取消时，将调用 on_abort 回调，已取消的任务会有 5s 时间用于清理现场并使用 {@link exit} 安全退出，

    否则执行器将直接终止运行脚本，并标记状态为 `'timed_out'`

    '''

    @property
    def aborted(self):
        return self._aborted

    def on_abort(self, func, args=[], kwargs={}):
        """
            监听前台的终止事件

            :params func 响应后的回调函数
            :params args 回调函数参数
            :params kwargs 回调函数参数
        """
        if func is not None:
            if self._aborted:
                func(*args, **kwargs)
            else:
                self.__abortedEvents += [func, args, kwargs]

    def exit(self, code):
        '''
            结束任务
            
            当任务未被取消时，分别被标记为 `'resolved'` 和 `'rejected'`；
            当任务被取消时，分别被标记为 `'aborted'` 和 `'rejected'`。
            
            调用此函数将导致进程直接终止，调用前请完成清理工作。
            
            :param code 程序退出码，运行成功返回 0，否则返回非 0
        '''
        sys.exit(code)

    def print(self, data):
        print(json.dumps(data, cls=MatlabDataEncoder), flush=True)

    def log(self, content, level='info', html=False, key=None):
        '''
            发送日志消息

            :param content 日志内容

            :param level 日志级别，可选值为 `'critical'`、`'error'`、`'warning'`、`'info'`、`'verbose'`、`'debug'`

            :param html 是否为 HTML 格式

            :param key 消息 key，用于在多个消息中引用同一消息实体，以便进行更新，或将指定 key 的消息放入容器

        '''
        self.print({
            "key": key,
            "version": 1,
            "verb": "replace",
            "type": "log",
            "data": {
                "level": level,
                'content': str(content),
                "html": html
            }
        })

    def container(self,
                  items=[],
                  layout={
                      'type': 'tabs',
                      'position': 'top'
                  },
                  key=None):
        '''
            
            发送分组消息

            :param items 分组成员，如 `[{title: 'item1', placeholder: 'Data loading', html: false, query: {type: 'message', key: 'item-1'}}]`

            :param layout 分组布局，如无特殊需求请直接使用 {@link tabsContainer} 和 {@link gridContainer}

            :param key 消息 key，用于在多个消息中引用同一消息实体，以便进行更新，或将指定 key 的消息放入容器

            @see {@link gridContainer}

            @see {@link tabsContainer}
    
        '''
        self.print({
            "key": key,
            "version": 1,
            "verb": "create",
            "type": "container",
            "data": {
                "layout": layout,
                "items": items
            }
        })

    def tabsContainer(self, items=[], position='top', key=None):
        '''
            发送 tabs 布局分组消息
        
            tabs 布局中，items[].title 为 tab 的标题
    
            :param items 分组成员，如 `[{'title': 'tab1', 'placeholder': 'Data loading', 'html': false, 'query': {'type': 'message', 'key': 'item-1'}}]`
            :param position tab 位置
            :param key 消息 key，用于在多个消息中引用同一消息实体，以便进行更新，或将指定 key 的消息放入容器
            @example
                >>> FunctionExecution.current.tabsContainer([
                        { 'title': 'tab1', 'placeholder': 'Data loading', 'html': false, 'query': { 'type': 'message', key: 'message-key-1' } },
                        { 'title': 'tab2', 'placeholder': 'Data loading', 'html': false, 'query': { 'type': 'message', 'key': 'message-key-2' } },
                    ])
                    #later
                    FunctionExecution.current.log('Content of tab1', 'info', false, 'message-key-1')
                    FunctionExecution.current.log('Content of tab2', 'info', false, 'message-key-2')
            @see {@link container}
            @see {@link gridContainer}
        '''
        self.container(items, {'type': 'tabs', 'position': position}, key)

    def gridContainer(
            self,
            item=[],
            grid="'item1 . item2' 1fr 'item1 item3 item4' 1fr / 1fr auto 2fr",
            key=None):
        '''
            发送 grid 布局分组消息

            *
            grid 布局中，items[].title 为 grid-area 名称
            *

            :param items 分组成员，如 `[{title: 'item1', placeholder: 'Data loading', html: false, query: {type: 'message', key: 'item-1'}}]`

            :param grid grid 布局说明，如 `'item1 . item2' 1fr 'item1 item3 item4' 1fr / 1fr auto 2fr`，见 https://developer.mozilla.org/en-US/docs/Web/CSS/grid-template

            :param key 消息 key，用于在多个消息中引用同一消息实体，以便进行更新，或将指定 key 的消息放入容器
            
            @example
                >>> FunctionExecution.current.gridContainer([
                        { 'title': 'item1', 'placeholder': 'Data loading', 'html': False, 'query': { 'type': 'message', 'key': 'message-key-1' } },
                        { title: 'item2', placeholder: 'Data loading', html: False, query: { type: 'message', key: 'message-key-2' } },
                        { title: 'item3', placeholder: 'Data loading', html: False, query: { type: 'message', key: 'message-key-3' } },
                    ], `'item1 item2' 1fr 'item3 item3' 2fr / 1fr 2fr`)
                    # later
                    FunctionExecution.current.log('Content of item1', 'info', False, 'message-key-1')
                    FunctionExecution.current.log('Content of item2', 'info', False, 'message-key-2')
                    FunctionExecution.current.log('Content of item3', 'info', False, 'message-key-3')
                    # You'll see the following grid:
                    #    1fr        2fr
                    # | item1 | C.of item2 | 1fr
                    # |       Content      | 2fr
                    # |      of item3      |
            @see {@link container}
            @see {@link tabsContainer}
        '''
        self.container(item, {'type': 'grid', 'grid': grid}, key)

    def progress(self, value=0, title='', key='progress-1'):
        '''
            发送进度信息
            
            :param value 当前进度值，取值范围 0~1

            :param title 进度标题

            :param key 消息 key，用于在多个消息中引用同一消息实体，以便进行更新，或将指定 key 的消息放入容器
        '''
        self.print({
            "key": key,
            "version": 1,
            "verb": "replace",
            "type": "progress",
            "data": {
                "value": value,
                "title": title
            }
        })

    def table(self, columns=[], title='', key=None, verb='replace'):
        '''
            发送表格信息
        
            :param columns 按列分组的表格内容

            :param title 表格标题

            :param key 消息 key，用于在多个消息中引用同一消息实体，以便进行更新，或将指定 key 的消息放入容器

            :param verb 特殊谓词，使用 `'append'` 和 `'prepend'` 在已有表格上追加或插入内容

            @example
            >>> FunctionExecution.current.table([
                    { 'name': 'col1', 'type': 'text', 'data': ['a', 'b', 'c'] },
                    { 'name': 'col2', 'type': 'number', 'data': [1, 2, 3] },
                ], 'My Data Table', 'table-1')
                #later
                FunctionExecution.current.table([
                    { 'name': 'col1', 'type': 'text', 'data': ['d'] },
                    { 'name': 'col2', 'type': 'number', 'data': [4] },
                ], 'My Data Table - Updated', 'table-1', 'append')
        '''
        self.print({
            "key": key,
            "version": 1,
            "verb": verb,
            "type": "table",
            "data": {
                "columns": columns,
                "title": title
            }
        })

    def __flat_map(self, data, target=None, prefix=""):
        if target is None:
            target = {}
        for k, v in data.items():
            if type(v) is dict:
                self.__flat_map(v, target, prefix + k + ".")
            else:
                target[prefix + k] = v
        return target

    def __plotlyDataToTrace(self, data):

        result = self.__flat_map(data)
        return result

    def plot(self, traces=[], layout={}, title='', key=None, verb='replace'):
        '''
            发送图表信息
        
            :param traces 图表数据

            :param layout 图表坐标轴信息

            :param title 图表标题

            :param key 消息 key，用于在多个消息中引用同一消息实体，以便进行更新，或将指定 key 的消息放入容器

            :param verb 特殊谓词，使用 `'append'` 和 `'prepend'` 在已有图表上追加或插入内容，使用 `'update'` 进行局部更新

            @example
                >>> FunctionExecution.current.plot([
                { 'name': 'trace1', 'type': 'scatter', 'x': [1, 2, 3], 'y': [1, 2, 3] },
                    { 'name': 'trace2', 'type': 'bar', 'x': [1, 2, 3], 'y': [1, 2, 3] },
                ], {
                    'xaxis': { 'title': 'x' },
                    'yaxis': { 'title': 'y' },
                }, 'My Plot', 'plot-1')
                #later
                FunctionExecution.current.plot([
                    { 'x': [4], 'y': [4] },
                    { 'x': [4], 'y': [4] },
                ], {}, 'My Plot - Updated', 'plot-1', 'append')
            @see traces 参见 {@link https://plotly.com/javascript/reference/index/ plotly} 文档
        '''

        result = {
            'key': key,
            'version': 1,
            'verb': verb,
            'type': 'plot',
            'data': {
                'title': title,
                'traces':
                [self.__plotlyDataToTrace(trace) for trace in traces],
                'xAxis': layout.get('xaxis', {}),
                'yAxis': layout.get('yaxis', {}),
            }
        }
        self.print(result)


# define an asynchronous iterator
import asyncio


class AsyncIterator():
    # constructor, define some state
    def __init__(self):
        self.counter = 0
 
    # create an instance of the iterator
    def __aiter__(self):
        return self
 
    # return the next awaitable
    async def __anext__(self):
        # check for no further items
        if self.counter >= 10:
            raise StopAsyncIteration
        # increment the counter
        self.counter += 1
        await asyncio.sleep(1)
        # return the counter value
        return self.counter

async def testAsync():
    # create an instance of the iterator
    asyncIterator = AsyncIterator()
    # iterate over the async iterator
    async for item in asyncIterator:
        print(item)

if __name__ == "__main__":
    asyncio.run(testAsync())


import os,sys
import time
sys.path.append(os.path.join(os.path.dirname(__file__), '..\\'))
import cloudpss 
import socketio
import threading
import struct
import json

class RealTimeSream(socketio.ClientNamespace):
    
    config = None
    messages = []
    series=[]
    outputNumericValue={}
    inputNumericValue={}
    
    end=False
    
    def __init__(self, namespace=None, *args, **kwargs):
        super().__init__(namespace)
        self.numericValueCallBack = kwargs.get('numericValueCallBack',None)
        self.chartCallBack = kwargs.get('chartCallBack',None)
        self.onStartJob = kwargs.get('onStartJob',None)
        setattr(self, 'on_data-panel', self.on_data_panel)
        setattr(self, 'on_start-job', self.on_start_job)
        setattr(self, 'on_real-time-data', self.on_chart_data)
        
        
        
        
    def on_connect(self):
        print('connect')
        # self.emit('start-job', { 'type': 'start-job', 'context': 'real-time-data' })
        # self.emit('real-time-data', { 'type': 'real-time-data', 'context': 'real-time-data' })
        # self.emit('data-panel', { 'type': 'data-panel', 'context': 'real-time-data' })
        pass

    def on_disconnect(self):
        print('disconnect')
        pass

    def on_data_panel(self, data):
        # self.numericValue=data
        virtual_input=self.config['virtual_input']
        virtual_output=self.config['virtual_output']
        display=self.config['display']
        
        d ={}
        for x,v in display.items():
            d[v]=x
        
        for i in range(0,virtual_input):
            self.inputNumericValue[d[i]]=data[i]
        for i in range(0,virtual_output): 
            self.outputNumericValue[d[virtual_input+i]]=data[virtual_input+i]
        if self.numericValueCallBack is not None:
            self.numericValueCallBack(self.inputNumericValue,self.outputNumericValue)
        
    def on_chart_data(self,msg):
        print(msg)

    def on_start_job(self,data):
        self.config =data
        if self.onStartJob is not None:
            self.onStartJob(self.config)
        print( 'on_start_job',self.config)
        
    
        
    def close(self):
        self.end =True
        self.disconnect()
        
    @staticmethod
    def connect(*args, **kwargs):
        sio = socketio.Client()
        stream = RealTimeSream('',*args, **kwargs)
        
        sio.register_namespace(stream)
        
        url = os.environ.get('CLOUDPSS_API_URL')
        
        if url is None:
            url = 'http://10.101.10.91/'
        
        
        print('connect to ',url,flush=True)
        sio.connect(url,socketio_path='/api/real-time/socket.io')
    
        thread = threading.Thread(target=sio.wait, args=())
        thread.setDaemon(True)
        thread.start()
        return stream

if __name__ == '__main__':
    # logging.basicConfig(level=logging.DEBUG)
    job = cloudpss.currentJob()
    
    remoteAddr= job.args.get('remoteAddr','http://10.101.10.91/')
    
    if not remoteAddr.startswith('http://'):
        remoteAddr='http://'+remoteAddr
    
    os.environ['CLOUDPSS_API_URL'] = remoteAddr
    
    
    
    def numericValueCallBack(inputNumericValue,outputNumericValue):
        # job.log(json.dumps(inputNumericValue),key='numeric')
        job.custom(inputNumericValue,'numeric')
        job.custom(outputNumericValue,'output')
        # print(json.dumps(inputNumericValue),flush=True)
        
    def chartCallBack(series):
        for serie in series:
            job.plot([serie],key=serie['name'])
        # job.plot(series,key='test1')
    
    def onStartJob(config):
        virtual_input=config['virtual_input']
        virtual_output=config['virtual_output']
        display=config['display']
        d ={}
        for x,v in display.items():
            d[v]=x
        items =[]
        for i in range(0,virtual_output): 
            items.append({'title': d[virtual_input+i], 'placeholder': 'Data loading', 'html': False, 'query': d[virtual_input+i]})
        
        job.container(items,key='output_container')
            # print(d[virtual_input+i],flush=True)
            # outputNumericValue[d[virtual_input+i]]=data[virtual_input+i]
        

    stream = RealTimeSream.connect(onStartJob=onStartJob,numericValueCallBack=numericValueCallBack,chartCallBack=chartCallBack)
    def exitCallback():
        stream.close()
        time.sleep(1)
        print('exit')
        job.exit(0)

    job.on_abort(exitCallback, args=(), kwargs={})
    
    print('start',flush=True)
    while not stream.end :
        time.sleep(1)
        pass


# coding=UTF-8
from cloudpss.ieslab import IESLabSimulation, IESLabPlan
from .verify import setToken
from .runner import Runner, Result, EMTResult, PowerFlowResult
from .model import Model, ModelRevision, ModelTopology
from .project import Project
from .utils import MatlabDataEncoder, DateTimeEncode
from . import function

from .function import FunctionExecution
from .version import __version__
__all__ = [
    'setToken', 'Model', 'ModelRevision', 'ModelTopology', 'Runner', 'Result',
    'PowerFlowResult', 'EMTResult', 'MatlabDataEncoder', 'DateTimeEncode',
    'function', 'Project', 'currentJob', 'IESLabSimulation', 'IESLabPlan','__version__'
]



def currentJob():
    """
        获取当前的 currentExecution 实例
    """
    return FunctionExecution.current()


import asyncio

async def set_after(fut, delay, value):
    print(1)
    # Sleep for *delay* seconds.
    await asyncio.sleep(delay)
    print(2)
    # Set *value* as a result of *fut* Future.
    fut.set_result(value)

async def main():
    # Get the current event loop.
    loop = asyncio.get_running_loop()

    # Create a new Future object.
    fut = loop.create_future()

    # Run "set_after()" coroutine in a parallel Task.
    # We are using the low-level "loop.create_task()" API here because
    # we already have a reference to the event loop at hand.
    # Otherwise we could have just used "asyncio.create_task()".
    loop.create_task(
        set_after(fut, 1, '... world'))

    print('hello ...')

    # Wait until *fut* has a result (1 second) and print it.
    print(await fut)

asyncio.run(main())


import os

import json
from ..utils import request, DateTimeEncode, MatlabDataEncoder
from deprecated import deprecated


class Args(dict):
    """
    参数类
    """
    def __getattr__(self, key):
        """
            __getattr__

        """
        return self[key]


@deprecated(version='3.0', reason="该类将在 5.0 版本移除，请使用 currentExecution 类代替")
class Job(object):
    """ 
        Function Job

    """
    def __init__(self, **kwargs):
        self.id = kwargs.pop('id', None)
        self.apiUrl = kwargs.pop('apiUrl', None)
        self.executorId = kwargs.pop('executorId', None)
        self.homeUrl = kwargs.pop('homeUrl', None)
        self.version = kwargs.pop('version', None)
        self.token = kwargs.pop('token', None)
        self._args = None

    @staticmethod
    def current():
        """获取当前任务

        Returns:
            Job: 任务类

        """
        id = os.environ.get('CLOUDPSS_JOB_ID', None)
        apiUrl = os.environ.get('CLOUDPSS_API_URL', None)
        executorId = os.environ.get('CLOUDPSS_EXECUTOR_ID', None)
        homeUrl = os.environ.get('CLOUDPSS_HOME_URL', None)
        token = os.environ.get('CLOUDPSS_TOKEN', None)
        version = os.environ.get('CLOUDPSS_STUDIO_VERSION', None)
        job = Job(id=id,
                  apiUrl=apiUrl,
                  executorId=executorId,
                  homeUrl=homeUrl,
                  version=version,
                  token=token)
        args = Job.loadArgs()
        job.args = args
        return job

    @staticmethod
    def loadArgs():
        """加载当前任务参数

        Returns:
            dict: 任务参数

        """
        args = Args()
        for k, v in os.environ.items():
            if k.startswith('CLOUDPSS_JOB_ARG-'):
                s = k.split('-')
                key = bytes.decode(bytes.fromhex(s[1]))
                # print(key, v, flush=True)

                args[key] = json.loads(v)
        return args

    @property
    def args(self):
        return self._args

    @args.setter
    def args(self, val):
        self._args = val

    def feedDog(self):
        """
            通知看门狗，当前程序还在运行，当程序 30s 内没有输出时，执行器将直接 kill 运行脚本
        """
        print("", flush=True)

    def progress(self, val, key='progress-1', title=''):
        """
            发送进度信息

            :params val: float 进度值 [0-1]
            :params key: str 目标, key 相同的数据会往同一个目标写入数据
            :params title: str 进度 标题

            >>> job.progress(0.2)
        """

        result = {
            'key': key,
            'version': 1,
            'verb': 'append',
            'type': 'progress',
            'data': {
                'title': title,
                'value': val
            }
        }
        print(json.dumps(result), flush=True)

    def message(self, msg, level='info', key=None, verb='create'):
        """
            发送日志信息

            :params msg: str 消息内容
            :params level: 消息类型 默认 info 可选：'critical' | 'error' | 'warning' | 'info' | 'verbose' | 'debug'
            :params key: str 目标, key 相同的数据会往同一个目标写入数据
            :params verb: 操作类型，可选  'create' | 'replace' | 'append' | 'prepend'

            >>> job.message('info message')
            >>> job.message('error message', level='error')


            在同一个目标显示消息
            >>> job.message('message1', key='log-1')
            >>> job.message('message2', key='log-1',verb='append')

            替换目标数据
            >>> job.message('message1', key='log-1')
            >>> job.message('message2', key='log-1',verb='replace')

        """
        result = {
            'key': key,
            'version': 1,
            'verb': verb,
            'type': 'log',
            'data': {
                'level': level,
                'content': str(msg)
            }
        }
        print(json.dumps(result), flush=True)

    def plot(self, key, traces, verb='append', layout={}, **kwargs):
        """
            发送绘图消息

            :params key: 目标， key 相同数据将显示到同一个图表上
            :params traces: Array 曲线数据组，
               >>>> [{
                    'name': str,  曲线名称
                    'type': 'scatter' | 'bar'  曲线类型
                    'x' : float[]   x 轴数据
                    'y' : float[]   y 轴数据
                }]
            :params title: str 图表标题
            :params xAxis: dict 坐标轴设置
            :params yAxis: dict 坐标轴设置
                >>> {
                    'title':str,   轴标题
                    'type' : 'linear' | 'log' | 'date',  轴类型
                    'range': [number, number] | 'auto'   显示范围
                }

            :params verb: str 消息内容 可选： 'create' | 'replace' | 'append' | 'prepend' | 'update'

            >>> job.plot('plot-1',[{'name':'t1','type':'scatter','x':[1,2],'y':[3,4]}])

        """

        result = {
            'key': key,
            'version': 1,
            'verb': verb,
            'type': 'plot',
            'data': dict({'traces': traces}, **layout)
        }
        print(json.dumps(result, cls=MatlabDataEncoder), flush=True)

    def table(self, key, columns, title='', verb='append', **kwargs):
        """
            发送表格数据

            :params key: str 目标, key 相同的数据会往同一个目标写入数据

            :params columns: Array 表格的列数据组
            >>> [{
                'name': str, 列名称
                'type': 'text' | 'html' | 'number',  列类型
                'data': unknown[]  列数据
            }]
            :params title: str 图表标题
            :params verb: 操作类型，可选  'create' | 'replace' | 'append' | 'prepend'

            >>> job.table('info message')
            >>> job.table('error message', level='error')

        """

        result = {
            'key': key,
            'version': 1,
            'verb': verb,
            'type': 'table',
            'data': {
                'title': title,
                'columns': columns
            }
        }

        print(json.dumps(result, cls=MatlabDataEncoder), flush=True)

    def terminate(self, status):
        """
            发送结束消息

            :params status: 结束状态 可以选 'resolved' | 'rejected' | 'aborted' | 'timed_out'

            >>> job.terminate('resolved')
        """

        result = {
            'version': 1,
            'verb': '',
            'type': 'terminate',
            'data': {
                'status': status,
            }
        }

        print(json.dumps(result), flush=True)

    def abort(self, data):
        """
            发送中止消息

            :params data: 消息数据

            >>> job.abort({})
        """
        result = {'version': 1, 'verb': '', 'type': 'abort', 'data': data}

        print(json.dumps(result), flush=True)

    def __flat_map(self, data, target=None, prefix=""):
        if target is None:
            target = {}
        for k, v in data.items():
            if type(v) is dict:
                self.__flat_map(v, target, prefix + k + ".")
            else:
                target[prefix + k] = v
        return target

    def plotLyDataToTrace(self, data):

        result = self.__flat_map(data)
        return result

    def print(self, message):
        print(json.dumps(message, cls=DateTimeEncode), flush=True)

    def container(self, data, key=None, verb=None):
        """
            发送消息组
            :params key: 图标索引 key
            :params verb: 操作类型，可选  'create' | 'replace' | 'append' | 'prepend'
            :params data: 分组数据
            
            {
                layout?:
                    | {
                        /** 浮窗显示，title 为 anchor（如：`/design/diagram/cells/canvas_0_16`），用于决定显示的内容 */
                        type: 'float';
                    }
                    | {
                        /** tab 页方式显示，title 为 tab 的标题 */
                        type: 'tabs';
                        position: 'left' | 'right' | 'top' | 'bottom';
                    }
                    | {
                        /** grid 布局显示，title 为 grid-area 名称 */
                        type: 'grid';
                        /** CSS grid 属性，如 `'item1 . item2' 1fr 'item1 item3 item4' 1fr / 1fr auto 2fr` */
                        grid: string;
                    };
                items: Array<{
                    /** 根据 `layout.type` 具有不同的功能 */
                    title: string;
                    /** 对应 key 数据不存在时显示 */
                    placeholder: string;
                    /** placeholder 是否使用 html */
                    html: boolean;
                    /** 获取用于填充的数据的方式 */
                    query?:
                        | {
                            /** 查找指定 key 的消息填充 */
                            type: 'message';
                            key: string;
                            /** 当此属性存在时，如未找到指定消息，向调用方发送该 payload 进行查询 */
                            signal?: MessagePayload;
                        }
                        | {
                            /** 通用 web api */
                            type: 'http';
                            method: string;
                            url: string;
                            headers: Record<string, string>;
                            body: unknown;
                            /** 从响应中获取消息的方法 */
                            picker?:
                                | string[] // 从响应体的路径 pick（lodash.get）
                                | ExpressionSource<unknown>; // 通过表达式获取（可用对象 $req, $res）
                        }
                        | {
                            /** 从对象存储获取指定对象填充 */
                            type: 'object-storage';
                            hash: string;
                        }
                        | {
                            /** 从对象存储获取指定对象填充 */
                            type: 'object-storage';
                            hash: string;
                        };
                }>;
            };
            >>> job.container(data,"key-c")
        """
        result = {
            'version': 1,
            "key": key,
            'verb': verb,
            'type': 'container',
            'data': data
        }
        print(result)
        print(json.dumps(result), flush=True)

    def gridContainer(self, items, grid=None, key=None, verb=None):
        """
            发送 类型为grid 的消息组

            :params items: 需要关联的其他图表消息 {'title':标题，'placeholder': 默认显示内容，key：需要关联的图表}
            :params tabsPosition: 操作类型，可选   'top' | 'bottom'

            :params key: 图标索引 key
            :params verb: 操作类型，可选  'create' | 'replace' | 'append' | 'prepend'
        """
        itemData = []
        s = ''
        for val in items:
            itemData.append({
                'title': val['title'],
                'placeholder': val.get('placeholder', "no data"),
                "query": {
                    'type': 'message',
                    'key': val['key'],
                } if val.get('key', None) is not None else None
            })
            s = s + '"{0}" 1fr '.format(val['title'])
        s = s + "/ auto"
        grid = grid if grid is not None else s
        data = {'items': itemData, 'layout': {'type': 'grid', 'grid': grid}}

        self.container(data, key, verb)

    def tabContainer(self, items, tabsPosition='top', key=None, verb=None):
        """
            发送 类型为grid 的消息组

            :params keys: 需要关联的其他图表列表
            :params tabsPosition: 操作类型，可选  'top' | 'bottom'

            :params key: 图标索引 key
            :params verb: 操作类型，可选  'create' | 'replace' | 'append' | 'prepend'
            

        """
        itemData = []
        for val in items:
            itemData.append({
                'title': val['title'],
                'placeholder': val.get('placeholder', "no data"),
                "query": {
                    'type': 'message',
                    'key': val['key'],
                } if val.get('key', None) is not None else None
            })
        data = {
            'items': itemData,
            'layout': {
                'type': 'tabs',
                'position': tabsPosition
            }
        }
        self.container(data, key, verb)



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
from cloudpss import __version__
import os

os.system('git tag '+__version__)

setup(
    name='cloudpss-hydrogen',
    version=__version__,
    keywords=("cloudpss", "cloudpss-sdk"),
    description='cloudpss sdk',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT Licence",
    url='https://www.cloudpss.net',
    author='cloudpss',
    author_email='zhangdaming@cloudpss.net',
    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    python_requires='>=3.7',
    install_requires=[
        'cffi', 'cryptography', 'cycler', 'pycparser', 'PyJWT', 'numpy',
        'PyYAML', 'requests', 'websocket-client', 'pytz', 'deprecated'
    ],
)



import os,sys
import time
sys.path.append(os.path.join(os.path.dirname(__file__), '..\\'))
import cloudpss 
import socketio
import threading
import struct
import json

class RealTimeSream(socketio.ClientNamespace):
    
    config = None
    messages = []
    series=[]
    outputNumericValue={}
    inputNumericValue={}
    
    end=False
    
    def __init__(self, namespace=None, *args, **kwargs):
        super().__init__(namespace)
        
        setattr(self, 'on_data-panel', self.on_data_panel)
        setattr(self, 'on_start-job', self.on_start_job)
        setattr(self, 'on_stop-job', self.on_stop_job)
        setattr(self, 'on_close-job', self.on_close_job)
        self.numericValueCallBack = kwargs.get('numericValueCallBack',None)
        self.chartCallBack = kwargs.get('chartCallBack',None)
        
        
        
        
    def on_connect(self):
        self.emit('start-job', { 'type': 'start-job', 'context': 'real-time-data' })
        pass

    def on_disconnect(self):
        pass

    def on_data_panel(self, data):
        # self.numericValue=data
        virtual_input=self.config['virtual_input']
        virtual_output=self.config['virtual_output']
        display=self.config['display']
        
        d ={}
        for x,v in display.items():
            d[v]=x
        
        for i in range(0,virtual_input):
            self.inputNumericValue[d[i]]=data[i]
        for i in range(0,virtual_output): 
            self.outputNumericValue[d[virtual_input+i]]=data[virtual_input+i]
        if self.numericValueCallBack is not None:
            self.numericValueCallBack(self.inputNumericValue,self.outputNumericValue)
    def on_start_job(self,data):
        self.config =data
        print( 'on_start_job',self.config)
    def on_stop_job(self,data):
        print('on_stop_job',flush=True)
        self.disconnect()
        self.end=True
    def on_close_job(self,data):
        print('on_close_job',flush=True)
        self.disconnect()
        self.end=True
    def on_recording(self,data):
        print( 'on_recording',data)
    def on_load_record_data(self,data):
        print( 'on_load_record_data',data)
    def on_real_time_data(self,msg):
        display=self.config['display']
        d ={}
        for x,v in display.items():
            d[v]=x
        data =struct.unpack('d'*int(len(msg)/8),msg)
        i = 0
        runTime =0
        self.series=[]
        channelLength=int(self.config['portNumber'])+1
        virtual_input=int(self.config['virtual_input'])
        for index in range(0,len(data)):
            val = data[index]
            if(index%channelLength==0):
                i=0
                runTime = val
            elif i>virtual_input:
                serieIndex = i - virtual_input - 1
                
                if len(self.series)-1<serieIndex:
                    serie=  {
                        'name':  d[i-1],
                        'type': 'scatter',
                        'x': [],
                        'y': []
                    }  
                    self.series.append(serie)
                else:
                    serie=self.series[serieIndex]
                
                serie['x'].append(runTime)
                serie['y'].append(val)
            i+=1
        if self.chartCallBack is not None:
            self.chartCallBack(self.series)

    def close(self):
        self.end =True
        self.disconnect()
        # self.emit('close-job', { 'type': 'close-job', 'context': 'close-job' })
    def write(self,data):
        print('write',data,flush=True)
        self.emit('send-message', data)
    @staticmethod
    def connect(*args, **kwargs):
        sio = socketio.Client()
        stream = RealTimeSream('',*args, **kwargs)
        
        sio.register_namespace(stream)
        
        url = os.environ.get('CLOUDPSS_API_URL')
        
        if url is None:
            url = 'http://10.101.10.91/'
        
        sio.connect(url,socketio_path='/api/real-time/socket.io')
    
        thread = threading.Thread(target=sio.wait, args=())
        thread.setDaemon(True)
        thread.start()
        return stream

if __name__ == '__main__':
    # logging.basicConfig(level=logging.DEBUG)
    job = cloudpss.currentJob()
    
    remoteAddr= job.args.get('remoteAddr','http://10.101.10.91/')
    
    inputKey= job.args.get('inputKey',None)
    inputValue= job.args.get('inputValue',None)
    
    print('inputKey',inputKey)
    print('inputValue',inputValue)
    if not remoteAddr.startswith('http://'):
        remoteAddr='http://'+remoteAddr
    
    
    
    os.environ['CLOUDPSS_API_URL'] = remoteAddr
    
    
    def exitCallback():
        stream.close()
        time.sleep(1)
        print('exit')
        job.exit(0)

    job.on_abort(exitCallback, args=(), kwargs={})
    

    stream = RealTimeSream.connect()
    stream.write({inputKey:float(inputValue)})
    time.sleep(1)
    stream.close()
   
    job.exit(0)


import os
import re
import io
import json
import yaml
import gzip
from copy import deepcopy

from .revision import ModelRevision
from .jobDefinitions import JOB_DEFINITIONS
from ..utils import request, fileLoad, graphql_request
from ..verify import userName

from cloudpss.runner.result import  PowerFlowResult, EMTResult, Result, IESResult
from cloudpss.runner.runner import  Runner


class Model(object):
    """
        CloudPSS工程类，用于处理加载后的工程数据

        实例变量说明：

        rid         项目的 rid

        name         项目的名称

        description  项目的描述

        revision    当前项目的版本信息

        configs     当前项目的所有参数方案

        jobs        当前项目的所有计算方案

        context     当前项目的上下文相关信息

    """
    context: dict
    jobs: list
    configs: list

    def __init__(self, model: dict = {}):
        """
            项目初始化
        """
        for k, v in model.items():
            if k == 'revision':
                self.revision = ModelRevision(v)

            else:
                self.__dict__[k] = v

    def __getitem__(self, attr):
        return super(Model, self).__getattribute__(attr)

    def toJSON(self):
        """
            类对象序列化为 dict
            :return: dict
        """
        model = {**self.__dict__, 'revision': self.revision.toJSON()}
        return model

    def getAllComponents(self):
        """
            获取实现

            :return: 所有元件信息

            >>> model.getAllComponents()
            {
                'canvas_0_2': Component 实例
            }
        """
        diagramImplement = self.revision.getImplements().getDiagram()
        if diagramImplement is None:
            raise ValueError('不存在拓扑实现')
        return diagramImplement.getAllComponents()
    
    def addComponent(self, definition, label, args, pins, canvas=None, position=None, size=None):
        """
        添加元件实现

        :return: Component

        >>>> model.addComponent(args)
        """
        diagramImplement = self.revision.getImplements().getDiagram()
        if diagramImplement is None:
            raise ValueError("不存在拓扑实现")
        return diagramImplement.addComponent(definition, label, args, pins, canvas, position, size)
    
    def removeComponent(self, key):
        """
        删除元件实现

        :return: boolean

        >>>> model.removeComponent(key)
        """
        diagramImplement = self.revision.getImplements().getDiagram()
        if diagramImplement is None:
            raise ValueError("不存在拓扑实现")
        return diagramImplement.removeComponent(key)
    
    def updateComponent(self, key, args):
        """
        更新元件实现

        :return: bool

        >>>> model.updateComponent(key, args)
        """
        diagramImplement = self.revision.getImplements().getDiagram()
        if diagramImplement is None:
            raise ValueError("不存在拓扑实现")
        return diagramImplement.updateComponent(key, args)


    def getComponentsByRid(self, rid: str):
        """
            通过指定元件类型获取元件

            :params str: 元件类型

            :type rid: str

            :return: 按照元件的 rid 过滤后的 dict<>

            >>> model.getComponentsByRid('model/CloudPSS/newInductorRouter')
            {
                'canvas_0_2': Component 实例
            }

        """

        v = self.getAllComponents()

        cells = {}
        for key, val in v.items():

            if not val.shape.startswith('diagram-component'):
                continue
            if val.definition == rid:
                cells[key] = val
        return cells

    def getComponentByKey(self, componentKey: str):
        """
            通过元件的 key 获取对应的元件

            :params key: key 元件的key

            :Return: Component 实例

            >>> model.getComponentByKey('canvas_0_757')
            Component 实例
        """

        v = self.getAllComponents()
        return v[componentKey]

    def getModelJob(self, name):
        """
            获取指定名称的计算方案

            :params Name: Name 参数名称

            :return: 同名计算方案数组

            >>> model.getModelJob('电磁暂态方案 1')
        """
        jobs = []

        for val in self.jobs:
            if val['name'] == name:
                jobs.append(val)

        return jobs

    def createJob(self, jobType: str, name):
        """
            创建一个计算方案
            创建出的方案默认不加入到项目中，需要加入请调用 addJob

            :params jobType:  方案类型 
                电磁暂态仿真方案 emtp
                移频电磁暂态仿真方案 sfemt
                潮流计算方案 powerFlow

            :return: 返回一个指定类型的计算方案

            >>> model.createJob('emtp','emtp job')
            计算方案
        """
        job = deepcopy(JOB_DEFINITIONS[jobType])
        job['name'] = name
        return job

    def addJob(self, job: dict):
        """
            将计算方案添加到工程中

            :params job:  计算方案 dict

            >>> job = model.createJob('emtp','emtp job')
                model.addJob(job)
        """

        self.jobs.append(job)

    def getModelConfig(self, name):
        """
            获取指定名称的参数方案

            :params name:  参数方案名称

            :return: 同名的方案数组

            >>> model.getModelConfig('参数方案 1')
        """
        configs = []

        for val in self.configs:
            if val['name'] == name:
                configs.append(val)

        return configs

    def createConfig(self, name):
        """
            创建一个参数方案
            根据项目的第一个参数方案生成一个方案
            创建出的方案默认不加入到项目中，需要加入请调用 addConfig
            :params name:  参数方案名称

            :return: 返回一个参数方案 dict

            >>> job = model.createConfig('my config')
                参数方案
        """

        config = deepcopy(self.configs[0])
        config['name'] = name
        return config

    def addConfig(self, config):
        """
            将参数方案添加到工程中

            :params config:  参数方案 dict

            >>> config = model.createConfig('my config')
                model.addConfig(config)
        """

        self.configs.append(config)
        return config

    @staticmethod
    def fetchMany(name=None, pageSize=10, pageOffset=0):
        """
            获取用户可以运行的项目列表

            :params name:  查询名称，模糊查询
            :params pageSize:  分页大小
            :params pageOffset:  分页开始位置

            :return: 按分页信息返回项目列表

            >>> data=Model.fetchMany()
            [
                {'rid': 'model/admin/share-test', 'name': '1234', 'description': '1234'}
                ...
            ]
        """

        r = request('GET',
                    'api/resources/type/model/permission/write',
                    params={
                        'page_offset': pageOffset,
                        'page_size': pageSize,
                        'search': name
                    })
        modelList = json.loads(r.text)

        return [{
            'rid': '/'.join([val['type'], val['owner'], val['key']]),
            'name': val['name'],
            'description': val['description']
        } for val in modelList]

    @staticmethod
    def fetch(rid):
        """
            获取项目

            :params rid:  项目 rid 

            :return: 返回一个项目实例

            >>> model=Model.fetch('model/Demo/test')

        """
        query = """
            query  t($rid:ResourceId!){
                model(input:{rid: $rid}) {
                    configs
                    context
                    description
                    jobs
                    name
                    rid
                    tags
                    revision {
                        author
                        documentation
                        graphic
                        hash
                        implements
                        message
                        parameters
                        pins
                        version
                    }
                }
            }
        """
        data = graphql_request(query, {'rid': rid})
        if 'errors' in data:
            raise Exception(data['errors'][0]['message'])
        return Model(data['data']['model'])

    def run(self, job=None, config=None, name=None, **kwargs):
        """

            调用仿真 

            :params job:  调用仿真时使用的计算方案，不指定将使用算例保存时选中的计算方案
            :params config:  调用仿真时使用的参数方案，不指定将使用算例保存时选中的参数方案
            :params name:  任务名称，为空时使用项目的参数方案名称和计算方案名称

            :return: 返回一个运行实例

            >>> runner=model.run(job,config,'')
            runner

        """
        if job is None:
            currentJob = self.context['currentJob']
            job = self.jobs[currentJob]
        if config is None:
            currentConfig = self.context['currentConfig']
            config = self.configs[currentConfig]
        return self.revision.run(job, config, name, self.rid, **kwargs)

    def iesSimulationRun(self, job=None, config=None, name=None, **kwargs):
        return self.run(job=job, config=config, name=name, kwargs=kwargs)

    @staticmethod
    def load(filePath):
        """
            加载本地项目文件

            :params file:  文件目录

            :return: 返回一个项目实例

            >>> model = Model.load('filePath')

        """

        if not os.path.exists(filePath):
            raise FileNotFoundError('未找到文件')
        data = fileLoad(filePath)
        return Model(data)

    @staticmethod
    def dump(model, file):
        """
            下载项目文件

            :params model:  项目
            :params file:  文件路径

            :return: 无

            >>> Model.dump(model,file)
        """

        data = model.toJSON()

        yamlData = yaml.dump(data)
        with gzip.open(file, 'w') as output:
            with io.TextIOWrapper(output, encoding='utf-8') as enc:
                enc.write(yamlData)

    def save(self, key=None):
        """
            保存/创建项目 

            key 不为空时如果远程存在相同的资源名称时将覆盖远程项目。
            key 为空时如果项目 rid 不存在则抛异常，需要重新设置 key。
            如果保存时，当前用户不是该项目的拥有者时，将重新创建项目，重建项目时如果参数的 key 为空将使用当前当前项目的 key 作为资源的 key ，当资源的 key 和远程冲突时保存失败

            :params: model 项目
            :params: key 资源 id 的唯一标识符，
            
            :return: 保存成功/保存失败
            
            >>> model.save(model)
                model.save(model,'newKey') # 另存为新的项目

        """
        username = userName()

        if key is not None:
            matchObj = re.match(r'^[-_A-Za-z0-9]+$', key, re.I | re.S)
            if matchObj:
                self.rid = 'model/' + username + '/' + key
                try:
                    r = request('GET', 'api/resources/' + self.rid)
                    return Model.update(self)
                except:
                    return Model.create(self)
            else:
                raise Exception('key 能包含字母数子和下划线')
        else:
            t = '(?<=/)\\S+(?=/)'
            owner = re.search(t, self.rid)
            if owner is None:
                raise Exception('rid 错误，请传入 key')
            elif owner[0] != username:
                rid = re.sub(t, username, self.rid)
                try:
                    r = request('GET', 'api/resources/' + self.rid)
                    return Model.create(self)
                except:
                    raise Exception(rid + ' 该资源已存在，无法重复创建,请修改 key')

        return Model.update(self)

    @staticmethod
    def create(model):
        """
            新建项目 

            :params: model 项目

            :return: 保存成功/保存失败

            >>> Model.create(model)
            保存成功
        """
        # Model.update(model)
        t = '(?<=/)\\S+(?=/)'
        username = userName()
        owner = re.search(t, model.rid)

        if owner is None:
            raise Exception('rid 错误，无法保存')
        elif owner[0] != username:
            raise Exception('rid 错误，无法保存')

        modelQuery = """
                    mutation($a:CreateModelInput!){createModel(input:$a){
                                         rid
                                }}
                """
        isPublic = model.context.get('auth', '') != 'private'
        isComponent = model.context.get('category', '') == 'component'
        publicRead = model.context.get('publicRead', '') != False
        auth = (65539 if publicRead else 65537) if isPublic else 0
        revision = ModelRevision.create(model.revision, model.revision.hash)

        return graphql_request(
            modelQuery, {
                'a': {
                    'rid': model.rid,
                    'revision': revision['hash'],
                    'context': model.context,
                    'configs': model.configs,
                    'jobs': model.jobs,
                    'name': model.name,
                    'description': model.description,
                    'tags': model.tags,
                    "permissions": {
                        "moderator": 1,
                        "member": 1,
                        "everyone": auth,
                    },
                }
            })

    @staticmethod
    def update(model):
        """
            更新项目 

            :params: model 项目

            :return: 保存成功/保存失败

            >>> Model.update(model)
        """

        t = '(?<=/)\\S+(?=/)'
        username = userName()
        owner = re.search(t, model.rid)

        if owner is None:
            raise Exception('rid 错误，无法保存')
        elif owner[0] != username:
            raise Exception('rid 错误，无法保存')

        modelQuery = """
                    mutation($a:UpdateModelInput!){updateModel(input:$a){
                                         rid
                                }}
                """
        isPublic = model.context.get('auth', '') != 'private'
        isComponent = model.context.get('category', '') == 'component'
        publicRead = model.context.get('publicRead', '') != False
        auth = (65539 if publicRead else 65537) if isPublic else 0
        revision = ModelRevision.create(model.revision, model.revision.hash)

        xVersion = int(float(os.environ.get('X_CLOUDPSS_VERSION', 4)))
        tags= {
            "replace":model.tags
        }
        if xVersion==3:
            tags=model.tags
        
        return graphql_request(
            modelQuery, {
                'a': {
                    'rid': model.rid,
                    'revision': revision['hash'],
                    'context': model.context,
                    'configs': model.configs,
                    'jobs': model.jobs,
                    'name': model.name,
                    'description': model.description,
                    'tags': tags,
                    "permissions": {
                        "moderator": 1,
                        "member": 1,
                        "everyone": auth,
                    },
                }
            })

    def fetchTopology(
        self,
        implementType=None,
        config=None,
        maximumDepth=None,
    ):
        """
            通过项目信息，获取当前项目对应的拓扑数据

            :params implementType:  实现类型
            :params config: config 项目参数, 不指定将使用算例保存时选中的参数方案
            :params maximumDepth:  最大递归深度，用于自定义项目中使用 diagram 实现元件展开情况

            :return:  一个拓扑实例

            >>> topology=model.fetchTopology()
                topology=model.fetchTopology(implementType='powerFlow',config=config) # 获取潮流实现的拓扑数据
                topology=model.fetchTopology(maximumDepth=2) # 获取仅展开 2 层的拓扑数据
        """

        if self.revision is not None:
            if implementType is None:
                implementType = 'emtp'
            if config is None:
                currentConfig = self.context['currentConfig']
                config = self.configs[currentConfig]
            return self.revision.fetchTopology(implementType, config,
                maximumDepth)
        return None

    def runEMT(self,job=None,config=None)->Runner[EMTResult]:
        '''
            运行 emtp 内核，如果当前 model 没有创建  Job 时报错，默认使用第一个计算方案，进行仿真。

            :param: job 计算方案名称，可选，字符串类型或者字典类型,默认使用第一个计算方案，如果同名使用最靠前一个
            :param: config 参数方案，可选，字符串类型或者字典类型,默认使用保存时选中的参数方案
            
            :return: runner Runner[EMTResult]
        '''
        if job is None:
            currentJob = self.context['currentJob']
            job = self.jobs[currentJob]
            if job['rid'] != 'job-definition/cloudpss/emtp':
                for j in self.jobs:
                    if j['rid'] == 'job-definition/cloudpss/emtp':
                        job = j
        if job is None:
            raise Exception("找不到电磁暂态运行的计算方案")
        if job['rid'] != 'job-definition/cloudpss/emtp':
            raise Exception("不是电磁暂态运行生成算法的计算方案")
        if config is None:
            currentConfig = self.context['currentConfig']
            config = self.configs[currentConfig]
        return self.run(job=job, config=config)

    def runSFEMT(self,job=None,config=None)->Runner[EMTResult]:
        '''
            运行 移频电磁暂态 内核，如果当前 model 没有创建  Job 时报错，默认使用第一个计算方案，进行仿真。

            :param: job 计算方案名称，可选，字符串类型或者字典类型,默认使用第一个计算方案，如果同名使用最靠前一个
            :param: config 参数方案，可选，字符串类型或者字典类型,默认使用保存时选中的参数方案

            :return: runner Runner[EMTResult]
        '''
        if job is None:
            currentJob = self.context['currentJob']
            job = self.jobs[currentJob]
            if job['rid'] != 'job-definition/cloudpss/sfemt':
                for j in self.jobs:
                    if j['rid'] == 'job-definition/cloudpss/sfemt':
                        job = j
        if job is None:
            raise Exception("找不到移频电磁暂态运行的计算方案")
        if job['rid'] != 'job-definition/cloudpss/sfemt':
            raise Exception("不是移频电磁暂态运行生成算法的计算方案")
        if config is None:
            currentConfig = self.context['currentConfig']
            config = self.configs[currentConfig]
        return self.run(job=job, config=config)

    def runPowerFlow(self,job=None,config=None)->Runner[PowerFlowResult]:
        '''
            运行 潮流 内核，如果当前 model 没有创建  Job 时报错，默认使用第一个计算方案，进行仿真。

            :param: job 计算方案名称，可选，字符串类型或者字典类型,默认使用第一个计算方案，如果同名使用最靠前一个
            :param: config 参数方案，可选，字符串类型或者字典类型,默认使用保存时选中的参数方案

            :return: runner Runner[PowerFlowResult]
        '''
        if job is None:
            currentJob = self.context['currentJob']
            job = self.jobs[currentJob]
            if job['rid'] != 'job-definition/cloudpss/power-flow':
                for j in self.jobs:
                    if j['rid'] == 'job-definition/cloudpss/power-flow':
                        job = j
        if job is None:
            raise Exception("找不到潮流内核运行的计算方案")
        if job['rid'] != 'job-definition/cloudpss/power-flow':
            raise Exception("不是潮流内核运行生成算法的计算方案")
        if config is None:
            currentConfig = self.context['currentConfig']
            config = self.configs[currentConfig]
        return self.run(job=job, config=config)

    def runThreePhasePowerFlow(self,job=None,config=None)->Runner[PowerFlowResult]:
        '''
            运行 三相不平衡潮流 内核，如果当前 model 没有创建  Job 时报错，默认使用第一个计算方案，进行仿真。
            
            :param: job 计算方案名称，可选，字符串类型或者字典类型,默认使用第一个计算方案，如果同名使用最靠前一个
            :param: config 参数方案，可选，字符串类型或者字典类型,默认使用保存时选中的参数方案

            :return: runner Runner[PowerFlowResult]
        '''
        if job is None:
            currentJob = self.context['currentJob']
            job = self.jobs[currentJob]
            if job['rid'] != 'job-definition/cloudpss/three-phase-powerFlow':
                for j in self.jobs:
                    if j['rid'] == 'job-definition/cloudpss/three-phase-powerFlow':
                        job = j
        if job is None:
            raise Exception("找不到三相不平衡潮流内核运行的计算方案")
        if job['rid'] != 'job-definition/cloudpss/three-phase-powerFlow':
            raise Exception("不是三相不平衡潮流内核运行生成算法的计算方案")
        if config is None:
            currentConfig = self.context['currentConfig']
            config = self.configs[currentConfig]
        return self.run(job=job, config=config)
    
    def runIESLoadPrediction(self,job=None,config=None)->Runner[IESResult]:
        '''
            运行 负荷预测方案 内核，如果当前 model 没有创建  Job 时报错，默认使用第一个计算方案，进行仿真。
            
            :param: job 计算方案名称，可选，字符串类型或者字典类型,默认使用第一个计算方案，如果同名使用最靠前一个
            :param: config 参数方案，可选，字符串类型或者字典类型,默认使用保存时选中的参数方案

            :return: runner Runner[IESResult]
        '''
        if job is None:
            currentJob = self.context['currentJob']
            job = self.jobs[currentJob]
            if job['rid'] != 'job-definition/ies/ies-load-prediction':
                for j in self.jobs:
                    if j['rid'] == 'job-definition/ies/ies-load-prediction':
                        job = j
        if job is None:
            raise Exception("找不到负荷预测方案内核运行的计算方案")
        if job['rid'] != 'job-definition/ies/ies-load-prediction':
            raise Exception("不是负荷预测方案内核运行生成算法的计算方案")
        if config is None:
            currentConfig = self.context['currentConfig']
            config = self.configs[currentConfig]
        return self.run(job=job, config=config)
    
    def runIESPowerFlow(self,job=None,config=None)->Runner[IESResult]:
        '''
            运行 时序潮流方案 内核，如果当前 model 没有创建  Job 时报错，默认使用第一个计算方案，进行仿真。
            
            :param: job 计算方案名称，可选，字符串类型或者字典类型,默认使用第一个计算方案，如果同名使用最靠前一个
            :param: config 参数方案，可选，字符串类型或者字典类型,默认使用保存时选中的参数方案

            :return: runner Runner[IESResult]
        '''
        if job is None:
            currentJob = self.context['currentJob']
            job = self.jobs[currentJob]
            if job['rid'] != 'job-definition/ies/ies-power-flow':
                for j in self.jobs:
                    if j['rid'] == 'job-definition/ies/ies-power-flow':
                        job = j
        if job is None:
            raise Exception("找不到时序潮流方案内核运行的计算方案")
        if job['rid'] != 'job-definition/ies/ies-power-flow':
            raise Exception("不是时序潮流方案内核运行生成算法的计算方案")
        if config is None:
            currentConfig = self.context['currentConfig']
            config = self.configs[currentConfig]
        return self.run(job=job, config=config)

    def runIESEnergyStoragePlan(self,job=None,config=None)->Runner[IESResult]:
        '''
            运行 储能规划方案 内核，如果当前 model 没有创建  Job 时报错，默认使用第一个计算方案，进行仿真。
            
            :param: job 计算方案名称，可选，字符串类型或者字典类型,默认使用第一个计算方案，如果同名使用最靠前一个
            :param: config 参数方案，可选，字符串类型或者字典类型,默认使用保存时选中的参数方案

            :return: runner Runner[IESResult]
        '''
        if job is None:
            currentJob = self.context['currentJob']
            job = self.jobs[currentJob]
            if job['rid'] != 'job-definition/ies/ies-energy-storage-plan':
                for j in self.jobs:
                    if j['rid'] == 'job-definition/ies/ies-energy-storage-plan':
                        job = j
        if job is None:
            raise Exception("找不到储能规划方案内核运行的计算方案")
        if job['rid'] != 'job-definition/ies/ies-energy-storage-plan':
            raise Exception("不是储能规划方案内核运行生成算法的计算方案")
        if config is None:
            currentConfig = self.context['currentConfig']
            config = self.configs[currentConfig]
        return self.run(job=job, config=config)




import json
from math import fabs
from ..utils.httprequests import request


class IESLabPlanResult(object):
    taskId = None

    def __init__(self, simulationId, taskId=None, **kwargs) -> None:
        """
            初始化
            获得taskId
        """
        self.simulationId = simulationId
        if taskId is not None:
            self.taskId = taskId
        else:
            self.taskId = self.__getLastTask()
        pass

    def __getLastTask(self):
        '''
            获取最后一次运行的taskID

            :return: string 类型
        '''
        try:
            r=request('GET','api/ieslab-plan/taskmanager/getSimuLastTasks',params={
                'simuid':self.simulationId
            })
            result= json.loads(r.text)
            return result['data'].get('task_id',None)
        except:
            return None

    def status(self):
        '''
            获取运行状态

            :return: boolean 类型
        '''
        log = self.GetLogs()
        for l in log:
            if l['data'] == 'run ends':
                return True
        return False

    def GetLogs(self):
        '''
            获取运行日志

            :return: dict 类型，代表日志信息
        '''
        if self.taskId is None:
            raise Exception('未开始运行')
        r = request('GET',
                    'api/ieslab-plan/taskmanager/getOptimizationLog',
                    params={
                        'simuid': self.simulationId,
                        'taskid': self.taskId
                    })
        result = json.loads(r.text)
        return result['msg']

    def GetPlanNum(self):
        '''
            获取当前result实例对应的优化方案数量
            
            :return: int类型，代表优化方案的数量
        '''
        if self.taskId is None:
            raise Exception('未开始运行')
        r = request('GET',
                    'api/ieslab-plan/taskmanager/getOptimizationResult',
                    params={
                        'simuid': self.simulationId,
                        'taskid': self.taskId,
                        'resultType': 0
                    })
        result = json.loads(r.text)
        return len(result['data'])

    def GetPlanInfo(self, planID):
        '''
            获取planID对应的优化方案的基础信息
            
            :param: planID int类型，表示优化方案的ID，数值位于 0~优化方案数量 之间
            
            :return: dict类型，代表该方案的基础信息，包括投资、运行成本、负荷总量等信息
        '''
        if self.taskId is None:
            raise Exception('未开始运行')
        r = request('GET',
                'api/ieslab-plan/taskmanager/getOptimizationResult',
                params={
                    'simuid': self.simulationId,
                    'taskid': self.taskId,
                    'resultType': 0
                })
        data = json.loads(r.text).get('data', [])
        result = data[planID].get('data', {}).get('data', [])[0]
        header = data[planID].get('data', {}).get('headerDesc', [])
        dict_result = {val.get('hearderName', ''): result.get(val.get('key', ''), '') for val in header}
        return dict_result


    def GetPlanConfiguration(self, planID):
        '''
            获取planID对应的优化方案的配置信息
            
            :param: planID int类型，表示优化方案的ID，数值位于 0~优化方案数量 之间

            :return: dict类型，代表该方案的配置信息，包括每种设备的选型配置、容量配置、成本等相关信息
        '''
        len = self.GetPlanNum()
        if int(planID) > len:
            raise Exception('计算方案id未产生')
        r = request('GET',
                    'api/ieslab-plan/taskmanager/getOptimizationResult',
                    params={
                        'simuid': self.simulationId,
                        'taskid': self.taskId,
                        "planId": planID,
                        'resultType': 1
                    })
        d = json.loads(r.text)
        result = {}
        for val in d['data']:
            result[val['u_name']] = val['data']['data']
        return result

    def GetComponentResult(self, planID, componentID, typicalDayName=''):
        '''
            获取planID对应的优化方案下componentID对应元件的运行信息
            
            :param: planID int 类型，表示优化方案的ID，数值位于 0~优化方案数量 之间
            :param: componentID str 类型，表示元件的标识符
            :param typicalDayName str 类型，代表典型日的名称

            :return: dict类型，代表该元件在不同典型日下的运行信息
        '''
        len = self.GetPlanNum()
        if int(planID) > len:
            raise Exception('计算方案id未产生')
        r = request('GET',
                    'api/ieslab-plan/taskmanager/getOptimizationResult',
                    params={
                        'simuid': self.simulationId,
                        'taskid': self.taskId,
                        'resultType': 3,
                        'typicalDay': typicalDayName,
                        "planId": planID,
                        'u_name': componentID
                    })
        d = json.loads(r.text)
        dict_result = dict()
        for val in d['data']:
            for k, v in val.items():
                dict_result[k] = v
        return dict_result

    def GetComponentTypiDays(self, planId, componentID):
        '''
            获取当前result实例对应的优化方案数量

            :param: planID int 类型，表示优化方案的ID，数值位于 0~优化方案数量 之间
            :param: componentID str 类型，表示元件的标识符
            
            :return: int类型，代表优化方案的数量
        '''
        if self.taskId is None:
            raise Exception('未开始运行')
        r = request('GET',
                    'api/ieslab-plan/taskmanager/getOptimizationResult',
                    params={
                        'simuid': self.simulationId,
                        'taskid': self.taskId,
                        "planId": planId,
                        'u_name': componentID,
                        'resultType': 2
                    })
        result = json.loads(r.text)
        return result['data']
    
    def getLastTaskResult(self):
        '''
            获取最后一次运行的taskID的运行结果

            :return: dict 类型
        '''
        r=request('GET','api/ieslab-plan/taskmanager/getSimuLastTasks',params={
            'simuid':self.simulationId
        })
        result= json.loads(r.text)
        return result
